@@ -31,22 +31,7 @@ ALLEGRO_AUDIO_STREAM *_al_load_flac_audio_stream_f(ALLEGRO_FILE* f,
 #endif
 
 #ifdef ALLEGRO_CFG_ACODEC_MODAUDIO
-ALLEGRO_AUDIO_STREAM *_al_load_mod_audio_stream(const char *filename,
-   size_t buffer_count, unsigned int samples);
-ALLEGRO_AUDIO_STREAM *_al_load_it_audio_stream(const char *filename,
-   size_t buffer_count, unsigned int samples);
-ALLEGRO_AUDIO_STREAM *_al_load_xm_audio_stream(const char *filename,
-   size_t buffer_count, unsigned int samples);
-ALLEGRO_AUDIO_STREAM *_al_load_s3m_audio_stream(const char *filename,
-   size_t buffer_count, unsigned int samples);
-ALLEGRO_AUDIO_STREAM *_al_load_mod_audio_stream_f(ALLEGRO_FILE *f,
-   size_t buffer_count, unsigned int samples);
-ALLEGRO_AUDIO_STREAM *_al_load_it_audio_stream_f(ALLEGRO_FILE *f,
-   size_t buffer_count, unsigned int samples);
-ALLEGRO_AUDIO_STREAM *_al_load_xm_audio_stream_f(ALLEGRO_FILE *f,
-   size_t buffer_count, unsigned int samples);
-ALLEGRO_AUDIO_STREAM *_al_load_s3m_audio_stream_f(ALLEGRO_FILE *f,
-   size_t buffer_count, unsigned int samples);
+bool _al_register_dumb_loaders(void);
 #endif
 
 #ifdef ALLEGRO_CFG_ACODEC_VORBIS
@@ -58,6 +43,14 @@ ALLEGRO_AUDIO_STREAM *_al_load_ogg_vorbis_audio_stream_f(ALLEGRO_FILE* file,
    size_t buffer_count, unsigned int samples);
 #endif
 
+#ifdef ALLEGRO_CFG_ACODEC_OPUS
+ALLEGRO_SAMPLE *_al_load_ogg_opus(const char *filename);
+ALLEGRO_SAMPLE *_al_load_ogg_opus_f(ALLEGRO_FILE *file);
+ALLEGRO_AUDIO_STREAM *_al_load_ogg_opus_audio_stream(const char *filename,
+   size_t buffer_count, unsigned int samples);
+ALLEGRO_AUDIO_STREAM *_al_load_ogg_opus_audio_stream_f(ALLEGRO_FILE* file,
+   size_t buffer_count, unsigned int samples);
+#endif
 
 
 #endif
