@@ -3,6 +3,7 @@
 
 #include "allegro5/allegro.h"
 #include "allegro5/allegro_image.h"
+#include "allegro5/internal/aintern.h"
 #include "allegro5/internal/aintern_convert.h"
 #include "allegro5/internal/aintern_exitfunc.h"
 #include "allegro5/internal/aintern_image.h"
@@ -220,6 +221,42 @@ public:
    }
 };
 
+static void load_indexed_data(Gdiplus::Bitmap *gdi_bmp,
+   ALLEGRO_BITMAP *a_bmp, uint32_t w, uint32_t h)
+{
+   Gdiplus::BitmapData *gdi_lock = new Gdiplus::BitmapData();
+   Gdiplus::Rect rect(0, 0, w, h);
+
+   if (!gdi_bmp->LockBits(&rect, Gdiplus::ImageLockModeRead,
+         PixelFormat8bppIndexed, gdi_lock))
+   {
+      ALLEGRO_LOCKED_REGION *a_lock = al_lock_bitmap(a_bmp,
+         ALLEGRO_PIXEL_FORMAT_SINGLE_CHANNEL_8, ALLEGRO_LOCK_WRITEONLY);
+
+      if (a_lock) {
+         unsigned char *in = (unsigned char *)gdi_lock->Scan0;
+         unsigned char *out = (unsigned char *)a_lock->data;
+
+         if (gdi_lock->Stride == a_lock->pitch) {
+            memcpy(out, in, h * gdi_lock->Stride);
+         }
+         else {
+            uint32_t rows = h;
+            while (rows--) {
+               memcpy(out, in, w);
+               in += gdi_lock->Stride;
+               out += a_lock->pitch;
+            }
+         }
+         al_unlock_bitmap(a_bmp);
+      }
+
+      gdi_bmp->UnlockBits(gdi_lock);
+   }
+
+   delete gdi_lock;
+}
+
 static void load_non_indexed_data(Gdiplus::Bitmap *gdi_bmp,
    ALLEGRO_BITMAP *a_bmp, uint32_t w, uint32_t h, bool premul)
 {
@@ -280,7 +317,7 @@ static void load_non_indexed_data(Gdiplus::Bitmap *gdi_bmp,
    delete gdi_lock;
 }
 
-ALLEGRO_BITMAP *_al_load_gdiplus_bitmap_f(ALLEGRO_FILE *fp)
+ALLEGRO_BITMAP *_al_load_gdiplus_bitmap_f(ALLEGRO_FILE *fp, int flags)
 {
    AllegroWindowsStream *s = new AllegroWindowsStream(fp);
    if (!s) {
@@ -293,12 +330,18 @@ ALLEGRO_BITMAP *_al_load_gdiplus_bitmap_f(ALLEGRO_FILE *fp)
    if (gdi_bmp) {
       const uint32_t w = gdi_bmp->GetWidth();
       const uint32_t h = gdi_bmp->GetHeight();
-      bool premul =
-         !(al_get_new_bitmap_flags() & ALLEGRO_NO_PREMULTIPLIED_ALPHA);
+      const PixelFormat pf = gdi_bmp->GetPixelFormat();
+      bool premul = !(flags & ALLEGRO_NO_PREMULTIPLIED_ALPHA);
+      bool keep_index = (flags & ALLEGRO_KEEP_INDEX);
 
       a_bmp = al_create_bitmap(w, h);
       if (a_bmp) {
-         load_non_indexed_data(gdi_bmp, a_bmp, w, h, premul);
+         if (pf == PixelFormat8bppIndexed && keep_index) {
+            load_indexed_data(gdi_bmp, a_bmp, w, h);
+         }
+         else {
+            load_non_indexed_data(gdi_bmp, a_bmp, w, h, premul);
+         }
       }
       delete gdi_bmp;
    }
@@ -308,14 +351,14 @@ ALLEGRO_BITMAP *_al_load_gdiplus_bitmap_f(ALLEGRO_FILE *fp)
    return a_bmp;
 }
 
-ALLEGRO_BITMAP *_al_load_gdiplus_bitmap(const char *filename)
+ALLEGRO_BITMAP *_al_load_gdiplus_bitmap(const char *filename, int flags)
 {
    ALLEGRO_BITMAP *bmp = NULL;
    ALLEGRO_FILE *fp;
 	
    fp = al_fopen(filename, "rb");		
    if (fp) {
-      bmp = _al_load_gdiplus_bitmap_f(fp);
+      bmp = _al_load_gdiplus_bitmap_f(fp, flags);
       al_fclose(fp);
    }
 
@@ -328,19 +371,19 @@ bool _al_save_gdiplus_bitmap_f(ALLEGRO_FILE *fp, const char *ident,
    CLSID encoder;
    int encoder_status = -1;
 
-   if (!strcmp(ident, ".bmp")) {
+   if (!_al_stricmp(ident, ".bmp")) {
       encoder_status = GetEncoderClsid(L"image/bmp", &encoder);
    }
-   else if (!strcmp(ident, ".jpg") || !strcmp(ident, ".jpeg")) {
+   else if (!_al_stricmp(ident, ".jpg") || !_al_stricmp(ident, ".jpeg")) {
       encoder_status = GetEncoderClsid(L"image/jpeg", &encoder);
    }
-   else if (!strcmp(ident, ".gif")) {
+   else if (!_al_stricmp(ident, ".gif")) {
       encoder_status = GetEncoderClsid(L"image/gif", &encoder);
    }
-   else if (!strcmp(ident, ".tif") || !strcmp(ident, ".tiff")) {
+   else if (!_al_stricmp(ident, ".tif") || !_al_stricmp(ident, ".tiff")) {
       encoder_status = GetEncoderClsid(L"image/tiff", &encoder);
    }
-   else if (!strcmp(ident, ".png")) {
+   else if (!_al_stricmp(ident, ".png")) {
       encoder_status = GetEncoderClsid(L"image/png", &encoder);
    }
 
