@@ -4,7 +4,10 @@
 
 #include "iio.h"
 
-ALLEGRO_BITMAP *_al_load_pcx_f(ALLEGRO_FILE *f)
+/* Do NOT simplify this to just (x), it doesn't work in MSVC. */
+#define INT_TO_BOOL(x)   ((x) != 0)
+
+ALLEGRO_BITMAP *_al_load_pcx_f(ALLEGRO_FILE *f, int flags)
 {
    ALLEGRO_BITMAP *b;
    int c;
@@ -15,6 +18,7 @@ ALLEGRO_BITMAP *_al_load_pcx_f(ALLEGRO_FILE *f)
    ALLEGRO_LOCKED_REGION *lr;
    unsigned char *buf;
    PalEntry pal[256];
+   bool keep_index;
    ASSERT(f);
 
    al_fgetc(f);                    /* skip manufacturer ID */
@@ -57,6 +61,8 @@ ALLEGRO_BITMAP *_al_load_pcx_f(ALLEGRO_FILE *f)
    if (!b) {
       return NULL;
    }
+   
+   keep_index = INT_TO_BOOL(flags & ALLEGRO_KEEP_INDEX);
 
    al_set_errno(0);
 
@@ -71,7 +77,12 @@ ALLEGRO_BITMAP *_al_load_pcx_f(ALLEGRO_FILE *f)
       buf = (unsigned char *)al_malloc(bytes_per_line * 3);
    }
 
-   lr = al_lock_bitmap(b, ALLEGRO_PIXEL_FORMAT_ABGR_8888_LE, ALLEGRO_LOCK_WRITEONLY);
+   if (bpp == 8 && keep_index) {
+      lr = al_lock_bitmap(b, ALLEGRO_PIXEL_FORMAT_SINGLE_CHANNEL_8, ALLEGRO_LOCK_WRITEONLY);
+   }
+   else {
+      lr = al_lock_bitmap(b, ALLEGRO_PIXEL_FORMAT_ABGR_8888_LE, ALLEGRO_LOCK_WRITEONLY);
+   }
    if (!lr) {
       al_free(buf);
       return NULL;
@@ -134,10 +145,15 @@ ALLEGRO_BITMAP *_al_load_pcx_f(ALLEGRO_FILE *f)
          char *dest = (char*)lr->data + y*lr->pitch;
          for (x = 0; x < width; x++) {
             int index = buf[y * width + x];
-            dest[x*4    ] = pal[index].r;
-            dest[x*4 + 1] = pal[index].g;
-            dest[x*4 + 2] = pal[index].b;
-            dest[x*4 + 3] = 255;
+            if (keep_index) {
+               dest[x] = index;
+            }
+            else {
+               dest[x*4    ] = pal[index].r;
+               dest[x*4 + 1] = pal[index].g;
+               dest[x*4 + 2] = pal[index].b;
+               dest[x*4 + 3] = 255;
+            }
          }
       }
    }
@@ -154,8 +170,6 @@ ALLEGRO_BITMAP *_al_load_pcx_f(ALLEGRO_FILE *f)
    return b;
 }
 
-/* Function: al_save_pcx_f
- */
 bool _al_save_pcx_f(ALLEGRO_FILE *f, ALLEGRO_BITMAP *bmp)
 {
    int c;
@@ -238,7 +252,7 @@ bool _al_save_pcx_f(ALLEGRO_FILE *f, ALLEGRO_BITMAP *bmp)
       return true;
 }
 
-ALLEGRO_BITMAP *_al_load_pcx(const char *filename)
+ALLEGRO_BITMAP *_al_load_pcx(const char *filename, int flags)
 {
    ALLEGRO_FILE *f;
    ALLEGRO_BITMAP *bmp;
@@ -248,7 +262,7 @@ ALLEGRO_BITMAP *_al_load_pcx(const char *filename)
    if (!f)
       return NULL;
 
-   bmp = _al_load_pcx_f(f);
+   bmp = _al_load_pcx_f(f, flags);
 
    al_fclose(f);
 
@@ -258,18 +272,35 @@ ALLEGRO_BITMAP *_al_load_pcx(const char *filename)
 bool _al_save_pcx(const char *filename, ALLEGRO_BITMAP *bmp)
 {
    ALLEGRO_FILE *f;
-   bool ret;
+   bool retsave;
+   bool retclose;
    ASSERT(filename);
 
    f = al_fopen(filename, "wb");
    if (!f)
       return false;
 
-   ret = _al_save_pcx_f(f, bmp);
+   retsave = _al_save_pcx_f(f, bmp);
 
-   al_fclose(f);
+   retclose = al_fclose(f);
 
-   return ret;
+   return retsave && retclose;
+}
+
+bool _al_identify_pcx(ALLEGRO_FILE *f)
+{
+   uint8_t x[4];
+   al_fread(f, x, 4);
+   
+   if (x[0] != 0x0a) // PCX must start with 0x0a
+      return false;
+   if (x[1] == 1 || x[1] > 5) // version must be 0, 2, 3, 4, 5
+      return false;
+   if (x[2] > 1) // compression must be 0 or 1
+      return false;
+   if (x[3] != 8) // only 8-bit PCX files supported by Allegro!
+      return false;
+   return true;
 }
 
 
