@@ -4,12 +4,7 @@
 
 #include <UIKit/UIAlertView.h>
 
-#include "allegro5/internal/aintern_iphone.h"
-
-
-/* In 5.1 this is exposed but not in 5.0 */
-UIView *_al_iphone_get_view(void);
-
+#include "allegro5/allegro_iphone_objc.h"
 
 bool _al_init_native_dialog_addon(void)
 {
@@ -46,8 +41,10 @@ bool _al_show_native_file_dialog(ALLEGRO_DISPLAY *display,
     al_signal_cond(button_pressed);
     al_unlock_mutex(mutex);
 }
--(void) createAlert:(id)alert {
-    [_al_iphone_get_view() addSubview:alert];
+- (void) createAlert:(NSArray*)array {
+    UIView *view = [array objectAtIndex:0];
+    UIAlertView *alert = [array objectAtIndex:1];
+    [view addSubview:alert];
     [alert show];
     [alert release];
 }
@@ -68,6 +65,9 @@ int _al_show_native_message_box(ALLEGRO_DISPLAY *display,
     AlertDelegate *delegate = [[AlertDelegate alloc]init];
     delegate.mutex = al_create_mutex();
     delegate.button_pressed = al_create_cond();
+    
+    // This needs to be done on the thread with the display due to TLS.
+    UIView *view = al_iphone_get_view(al_get_current_display());
 
     UIAlertView *alert = [[UIAlertView alloc]
                           initWithTitle:title
@@ -77,9 +77,8 @@ int _al_show_native_message_box(ALLEGRO_DISPLAY *display,
                           otherButtonTitles:nil];
 
     [delegate performSelectorOnMainThread:@selector(createAlert:)
-                               withObject:alert
-                            waitUntilDone:YES];
-    
+                               withObject:@[view,alert]
+                            waitUntilDone:YES];    
 
     al_lock_mutex(delegate.mutex);
     al_wait_cond(delegate.button_pressed, delegate.mutex);
@@ -108,3 +107,57 @@ void _al_append_native_text_log(ALLEGRO_NATIVE_DIALOG *textlog)
 {
     (void) textlog;
 }
+
+bool _al_init_menu(ALLEGRO_MENU *menu)
+{
+    (void) menu;
+    return false;
+}
+
+bool _al_init_popup_menu(ALLEGRO_MENU *menu)
+{
+    (void) menu;
+    return false;
+}
+
+bool _al_insert_menu_item_at(ALLEGRO_MENU_ITEM *item, int i)
+{
+    (void) item;
+    (void) i;
+    return false;
+}
+
+bool _al_destroy_menu_item_at(ALLEGRO_MENU_ITEM *item, int i)
+{
+    (void) item;
+    (void) i;
+    return false;
+}
+
+bool _al_update_menu_item_at(ALLEGRO_MENU_ITEM *item, int i)
+{
+    (void) item;
+    (void) i;
+    return false;
+}
+
+bool _al_show_display_menu(ALLEGRO_DISPLAY *display, ALLEGRO_MENU *menu)
+{
+    (void) display;
+    (void) menu;
+    return false;
+}
+
+bool _al_hide_display_menu(ALLEGRO_DISPLAY *display, ALLEGRO_MENU *menu)
+{
+    (void) display;
+    (void) menu;
+    return false;
+}
+
+bool _al_show_popup_menu(ALLEGRO_DISPLAY *display, ALLEGRO_MENU *menu)
+{
+    (void) display;
+    (void) menu;
+    return false;
+}
\ No newline at end of file
