@@ -16,18 +16,11 @@ struct DISPLAY_MENU
 
 static _AL_VECTOR display_menus = _AL_VECTOR_INITIALIZER(DISPLAY_MENU);
 
-/* MENU_ID keeps track of menu / id pairs to help determine which
- * menu belongs to an id.
- */
-typedef struct MENU_ID MENU_ID;
-
-struct MENU_ID
-{
-   ALLEGRO_MENU *menu;
-   uint16_t id;
-};
-
-static _AL_VECTOR menu_ids  = _AL_VECTOR_INITIALIZER(MENU_ID);
+/* The unique id. This is used to reverse lookup menus.
+ * The primarily need for this arises from Windows, which cannot store
+ * ALLEGRO_MENU_ID's wholesale.*/
+static uint16_t unique_id;
+static _AL_VECTOR menu_ids  = _AL_VECTOR_INITIALIZER(_AL_MENU_ID);
 
 /* The default event source is used with any menu that does not have
    its own event source enabled. */
@@ -42,6 +35,17 @@ static ALLEGRO_MENU_ITEM *interpret_menu_id_param(ALLEGRO_MENU **menu, int *id);
 static ALLEGRO_MENU_INFO *parse_menu_info(ALLEGRO_MENU *parent, ALLEGRO_MENU_INFO *info);
 static bool set_menu_display_r(ALLEGRO_MENU *menu, ALLEGRO_MENU_ITEM *item, int index, void *extra);
 
+/* True if the id is actually unique.
+ */
+static bool get_unique_id(uint16_t* id)
+{
+   if (unique_id + 1 == UINT16_MAX) {
+      return false;
+   }
+   *id = unique_id++;
+   return true;
+}
+
 /* The menu item owns the icon bitmap. It is converted to a memory bitmap
  * when set to make sure any system threads will be able to read the data. 
  */
@@ -62,6 +66,9 @@ static ALLEGRO_MENU_ITEM *create_menu_item(char const *title, uint16_t id, int f
 {
    ALLEGRO_MENU_ITEM *item = al_calloc(1, sizeof(*item));
    if (!item) return NULL;
+   if (!get_unique_id(&item->unique_id)) {
+      return NULL;
+   }
    
    if (flags & ALLEGRO_MENU_ITEM_CHECKED)
       flags |= ALLEGRO_MENU_ITEM_CHECKBOX;
@@ -150,12 +157,12 @@ static void destroy_menu_item(ALLEGRO_MENU_ITEM *item)
 
             /* Remove the command from the look-up vector. */
             if (item->id != 0) {
-               MENU_ID *menu_id;
+               _AL_MENU_ID *menu_id;
                size_t j;
 
                for (j = 0; j < _al_vector_size(&menu_ids); ++j) {
-                  menu_id = (MENU_ID *) _al_vector_ref(&menu_ids, j);
-                  if (menu_id->menu == item->parent && menu_id->id == item->id) {
+                  menu_id = (_AL_MENU_ID *) _al_vector_ref(&menu_ids, j);
+                  if (menu_id->menu == item->parent && menu_id->unique_id == item->unique_id) {
                      _al_vector_delete_at(&menu_ids, j);
                      break;
                   }
@@ -375,7 +382,7 @@ int al_insert_menu_item(ALLEGRO_MENU *parent, int pos, char const *title,
 {
    ALLEGRO_MENU_ITEM *item;
    ALLEGRO_MENU_ITEM **slot;
-   MENU_ID *menu_id;
+   _AL_MENU_ID *menu_id;
    size_t i;
 
    ASSERT(parent);
@@ -426,7 +433,8 @@ int al_insert_menu_item(ALLEGRO_MENU *parent, int pos, char const *title,
 
    if (id) {
       /* Append the menu's ID to the search vector */
-      menu_id = (MENU_ID *) _al_vector_alloc_back(&menu_ids);
+      menu_id = (_AL_MENU_ID *) _al_vector_alloc_back(&menu_ids);
+      menu_id->unique_id = item->unique_id;
       menu_id->id = id;
       menu_id->menu = parent;
    }
@@ -809,16 +817,16 @@ ALLEGRO_MENU *al_remove_display_menu(ALLEGRO_DISPLAY *display)
 /* Tries to find the menu that has a child with the given id. If display
  * is not NULL, then it must also match. The first match is returned.
  */
-ALLEGRO_MENU *_al_find_parent_menu_by_id(ALLEGRO_DISPLAY *display, uint16_t id)
+_AL_MENU_ID *_al_find_parent_menu_by_id(ALLEGRO_DISPLAY *display, uint16_t unique_id)
 {
-   MENU_ID *menu_id;
+   _AL_MENU_ID *menu_id;
    size_t i;
 
    for (i = 0; i < _al_vector_size(&menu_ids); ++i) {
-      menu_id = (MENU_ID *) _al_vector_ref(&menu_ids, i);
-      if (menu_id->id == id) {
+      menu_id = (_AL_MENU_ID *) _al_vector_ref(&menu_ids, i);
+      if (menu_id->unique_id == unique_id) {
          if (!display || menu_id->menu->display == display) {
-            return menu_id->menu;
+            return menu_id;
          }
       }
    }
@@ -831,22 +839,22 @@ ALLEGRO_MENU *_al_find_parent_menu_by_id(ALLEGRO_DISPLAY *display, uint16_t id)
  * and the user is using non-unique ids, it won't know which display actually
  * triggered the menu click.
  */
-bool _al_emit_menu_event(ALLEGRO_DISPLAY *display, uint16_t id)
+bool _al_emit_menu_event(ALLEGRO_DISPLAY *display, uint16_t unique_id)
 {
    ALLEGRO_EVENT event;
-   ALLEGRO_MENU *menu = NULL;
+   _AL_MENU_ID *menu_id = NULL;
    ALLEGRO_EVENT_SOURCE *source = al_get_default_menu_event_source();
 
-   if (id == 0)
+   /* try to find the menu that triggered the event */
+   menu_id = _al_find_parent_menu_by_id(display, unique_id);
+
+   if (menu_id->id == 0)
       return false;
 
-   /* try to find the menu that triggered the event */
-   menu = _al_find_parent_menu_by_id(display, id);
-
-   if (menu) {
+   if (menu_id) {
       /* A menu was found associated with the id. See if it has an
        * event source associated with it, and adjust "source" accordingly. */
-      ALLEGRO_MENU *m = menu;
+      ALLEGRO_MENU *m = menu_id->menu;
       while (true) {
          if (m->is_event_source) {
             source = &m->es;
@@ -864,9 +872,9 @@ bool _al_emit_menu_event(ALLEGRO_DISPLAY *display, uint16_t id)
    }
 
    event.user.type = ALLEGRO_EVENT_MENU_CLICK;
-   event.user.data1 = id;
+   event.user.data1 = menu_id->id;
    event.user.data2 = (intptr_t) display;
-   event.user.data3 = (intptr_t) menu;
+   event.user.data3 = (intptr_t) menu_id->menu;
       
    al_emit_user_event(source, &event, NULL);
 
