@@ -119,6 +119,7 @@ void _al_shutdown_native_dialog_addon(void)
 bool _al_show_native_file_dialog(ALLEGRO_DISPLAY *display,
                                  ALLEGRO_NATIVE_DIALOG *fd)
 {
+    NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
     (void)display;
     
     [ALLEGFileDialog performSelectorOnMainThread: @selector(displayFileDialog:)
@@ -126,6 +127,8 @@ bool _al_show_native_file_dialog(ALLEGRO_DISPLAY *display,
                                 waitUntilDone: YES];
     _al_osx_clear_mouse_state();
     
+    [pool release];
+
     return true;
 }
 
@@ -169,12 +172,14 @@ int _al_show_native_message_box(ALLEGRO_DISPLAY *display,
                                 ALLEGRO_NATIVE_DIALOG *fd)
 {
     (void)display;
-    @autoreleasepool {
+    {
+        NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
         NSValue* fdValue = [NSValue valueWithPointer:fd];
         
         [ALLEGAlertWrapper performSelectorOnMainThread: @selector(displayAlert:)
                                          withObject: fdValue
                                       waitUntilDone: YES];
+        [pool release];
     }
     _al_osx_clear_mouse_state();
     return fd->mb_pressed_button;
@@ -353,7 +358,7 @@ void _al_append_native_text_log(ALLEGRO_NATIVE_DIALOG *textlog)
     NSMutableArray * _items;
 }
 +(ALLEGTargetManager*) sharedManager;
--(instancetype) init;
+-(id) init;
 -(void) dealloc;
 -(void) setMenu: (NSMenu*) menu forWindow:(NSWindow*) window;
 @end
@@ -372,7 +377,7 @@ void _al_append_native_text_log(ALLEGRO_NATIVE_DIALOG *textlog)
     NSMenu* _menu;
 }
 -(NSMenu*) menu;
--(instancetype) initWithMenu:(ALLEGRO_MENU*) amenu; // Designated initializer
+-(id) initWithMenu:(ALLEGRO_MENU*) amenu; // Designated initializer
 -(NSMenu*) menu;
 -(void) show;
 -(void) showPopup;
@@ -535,7 +540,7 @@ bool _al_show_popup_menu(ALLEGRO_DISPLAY *display, ALLEGRO_MENU *amenu)
     }
 }
 // Create target with ALLEGRO_MENU bound to it.
-- (instancetype)initWithMenu:(ALLEGRO_MENU*) source_menu
+- (id)initWithMenu:(ALLEGRO_MENU*) source_menu
 {
     self = [super init];
     if (self) {
@@ -546,7 +551,7 @@ bool _al_show_popup_menu(ALLEGRO_DISPLAY *display, ALLEGRO_MENU *amenu)
     }
     return self;
 }
--(instancetype) init
+-(id) init
 {
     /* This isn't a valid initializer */
     return nil;
@@ -580,12 +585,13 @@ bool _al_show_popup_menu(ALLEGRO_DISPLAY *display, ALLEGRO_MENU *amenu)
         if (aitim->flags & ALLEGRO_MENU_ITEM_CHECKBOX) {
             aitim->flags ^= ALLEGRO_MENU_ITEM_CHECKED;
         }
-        _al_emit_menu_event(aitim->parent->display, aitim->id);
+        _al_emit_menu_event(aitim->parent->display, aitim->unique_id);
     }
 }
 // Insert an item, keep the NSMenu in sync
 -(void) insertItem:(ALLEGRO_MENU_ITEM*) aitem atIndex: (int) index
 {
+    NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
     NSMenuItem* item = [self buildMenuItemFor:aitem];
     [self.menu insertItem:item atIndex:index];
     if (aitem->popup) {
@@ -593,23 +599,28 @@ bool _al_show_popup_menu(ALLEGRO_DISPLAY *display, ALLEGRO_MENU *amenu)
         [[sub menu] setTitle:[item title]];
         [item setSubmenu:[sub menu]];
     }
+    [pool release];
 }
 // Update an item (caption only, see -validateMenuItem: )
 -(void) updateItem:(ALLEGRO_MENU_ITEM *)aitem atIndex:(int)index
 {
+    NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
     (void) index;
     NSMenuItem* item = [self itemForAllegroItem:aitem];
     NSMutableString* caption = [NSMutableString stringWithUTF8String:al_cstr(aitem->caption)];
     NSString* key = extract_accelerator(caption);
     [item setTitle:caption];
     [item setKeyEquivalent:key];
+    [pool release];
 }
 // Remove an item, keep the NSMenu in sync
 -(void) destroyItem:(ALLEGRO_MENU_ITEM *)aitem atIndex:(int)index
 {
+    NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
     (void) index;
     NSMenuItem* item = [self itemForAllegroItem:aitem];
     [item.menu removeItem:item];
+    [pool release];
 }
 // Show the menu on the main application menu bar
 -(void) show
@@ -657,7 +668,7 @@ static ALLEGTargetManager* _sharedmanager = nil;
     return _sharedmanager;
 }
 // Set up and register for notifications
--(instancetype) init
+-(id) init
 {
     self = [super init];
     if (self) {
@@ -712,9 +723,10 @@ static ALLEGTargetManager* _sharedmanager = nil;
 // Link a menu with a window, replace any existing link
 -(void) setMenu:(NSMenu *)menu forWindow:(NSWindow *)window
 {
+    NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
     NSUInteger index = [self indexForWindow:window];
     if (menu) {
-        NSDictionary* newentry = @{@"menu":menu, @"window":window};
+        NSDictionary* newentry = [NSDictionary dictionaryWithObjectsAndKeys:menu, @"menu", window, @"window", nil];
         if (index == NSNotFound) {
             [self->_items addObject: newentry];
         }
@@ -727,5 +739,6 @@ static ALLEGTargetManager* _sharedmanager = nil;
             [self->_items removeObjectAtIndex:index];
         }
     }
+    [pool release];
 }
 @end // ALLEGTargetManager
