@@ -1,12 +1,25 @@
 #include <stdio.h>
 #include <stdarg.h>
 
+#ifdef ALLEGRO_ANDROID
+   #include "allegro5/allegro_android.h"
+#endif
+
+void init_platform_specific(void);
 void abort_example(char const *format, ...);
 void open_log(void);
 void open_log_monospace(void);
 void close_log(bool wait_for_user);
 void log_printf(char const *format, ...);
 
+void init_platform_specific(void)
+{
+#ifdef ALLEGRO_ANDROID
+   al_install_touch_input();
+   al_android_set_apk_file_interface();
+#endif
+}
+
 #ifdef ALLEGRO_POPUP_EXAMPLES
 
 #include "allegro5/allegro_native_dialog.h"
@@ -99,8 +112,16 @@ void log_printf(char const *format, ...)
 {
    va_list args;
    va_start(args, format);
+   #ifdef ALLEGRO_ANDROID
+   char x[1024];
+   vsnprintf(x, sizeof x, format, args);
+   ALLEGRO_TRACE_CHANNEL_LEVEL("log", 1)(x);
+   #else
    vprintf(format, args);
+   #endif
    va_end(args);
 }
 
 #endif
+
+/* vim: set sts=3 sw=3 et: */
