@@ -6,6 +6,8 @@
 
 #include "common.c"
 
+ALLEGRO_DEBUG_CHANNEL("main")
+
 #define FPS 60
 #define MAX_SPRITES 1024
 
@@ -14,6 +16,7 @@ typedef struct Sprite {
 } Sprite;
 
 char const *text[] = {
+   "H - toggle held drawing",
    "Space - toggle use of textures",
    "B - toggle alpha blending",
    "Left/Right - change bitmap size",
@@ -27,13 +30,13 @@ struct Example {
    int blending;
    ALLEGRO_DISPLAY *display;
    ALLEGRO_BITMAP *mysha, *bitmap;
+   bool hold_bitmap_drawing;
    int bitmap_size;
    int sprite_count;
    bool show_help;
    ALLEGRO_FONT *font;
-    
-   bool mouse_down;
-   int last_x, last_y;
+
+   int t;
 
    ALLEGRO_COLOR white;
    ALLEGRO_COLOR half_white;
@@ -165,6 +168,12 @@ static void update(void)
    int i;
    for (i = 0; i < example.sprite_count; i++)
       sprite_update(example.sprites + i);
+
+   example.t++;
+   if (example.t == 60) {
+      ALLEGRO_DEBUG("tick");
+      example.t = 0;
+   }
 }
 
 static void redraw(void)
@@ -193,15 +202,24 @@ static void redraw(void)
    else if (example.blending == 3)
       al_set_blender(ALLEGRO_ADD, ALLEGRO_ONE, ALLEGRO_ZERO);
 
+   if (example.hold_bitmap_drawing) {
+      al_hold_bitmap_drawing(true);
+   }
    for (i = 0; i < example.sprite_count; i++) {
       Sprite *s = example.sprites + i;
       al_draw_tinted_bitmap(example.bitmap, tint, s->x, s->y, 0);
    }
+   if (example.hold_bitmap_drawing) {
+      al_hold_bitmap_drawing(false);
+   }
 
    al_set_blender(ALLEGRO_ADD, ALLEGRO_ONE, ALLEGRO_INVERSE_ALPHA);
    if (example.show_help) {
-      for (i = 0; i < 5; i++)
-         al_draw_text(example.font, example.white, 0, h - 5 * fh + i * fh, 0, text[i]);
+      int dh = fh * 3.5;
+      for (i = 5; i >= 0; i--) {
+         al_draw_text(example.font, example.white, 0, h - dh, 0, text[i]);
+         dh += fh * 6;
+      }
    }
 
    al_draw_textf(example.font, example.white, 0, 0, 0, "count: %d",
@@ -221,7 +239,7 @@ static void redraw(void)
    
 }
 
-int main(void)
+int main(int argc, char **argv)
 {
    ALLEGRO_TIMER *timer;
    ALLEGRO_EVENT_QUEUE *queue;
@@ -229,7 +247,12 @@ int main(void)
    int w = 640, h = 480;
    bool done = false;
    bool need_redraw = true;
+   bool background = false;
    example.show_help = true;
+   example.hold_bitmap_drawing = false;
+
+   (void)argc;
+   (void)argv;
 
    if (!al_init()) {
       abort_example("Failed to init Allegro.\n");
@@ -240,30 +263,38 @@ int main(void)
    }
 
    al_init_font_addon();
+   init_platform_specific();
 
    al_get_num_video_adapters();
    
    al_get_monitor_info(0, &info);
-   if (info.x2 - info.x1 < w)
-      w = info.x2 - info.x1;
-   if (info.y2 - info.y1 < h)
-      h = info.y2 - info.y1;
+
+   #if defined ALLEGRO_CFG_OPENGLES
+   al_set_new_display_flags(ALLEGRO_FULLSCREEN_WINDOW);
+   #endif
+   al_set_new_display_option(ALLEGRO_SUPPORTED_ORIENTATIONS,
+                             ALLEGRO_DISPLAY_ORIENTATION_ALL, ALLEGRO_SUGGEST);
    example.display = al_create_display(w, h);
    if (!example.display) {
       abort_example("Error creating display.\n");
    }
 
+   w = al_get_display_width(example.display);
+   h = al_get_display_height(example.display);
+
    if (!al_install_keyboard()) {
       abort_example("Error installing keyboard.\n");
    }
-    
+  
    if (!al_install_mouse()) {
       abort_example("Error installing mouse.\n");
    }
 
-   example.font = al_load_font("data/fixed_font.tga", 0, 0);
+   al_install_touch_input();
+
+   example.font = al_create_builtin_font();
    if (!example.font) {
-      abort_example("Error loading data/fixed_font.tga\n");
+      abort_example("Error creating builtin font\n");
    }
 
    example.mysha = al_load_bitmap("data/mysha256x256.png");
@@ -285,14 +316,20 @@ int main(void)
    al_register_event_source(queue, al_get_keyboard_event_source());
    al_register_event_source(queue, al_get_mouse_event_source());
    al_register_event_source(queue, al_get_timer_event_source(timer));
+   
+   if (al_install_touch_input())
+      al_register_event_source(queue, al_get_touch_input_event_source());
    al_register_event_source(queue, al_get_display_event_source(example.display));
 
    al_start_timer(timer);
 
    while (!done) {
+      float x, y;
       ALLEGRO_EVENT event;
+      w = al_get_display_width(example.display);
+      h = al_get_display_height(example.display);
 
-      if (need_redraw && al_is_event_queue_empty(queue)) {
+      if (!background && need_redraw && al_is_event_queue_empty(queue)) {
          double t = -al_get_time();
          add_time();
          al_clear_to_color(al_map_rgb_f(0, 0, 0));
@@ -332,66 +369,80 @@ int main(void)
                if (example.blending == 4)
                   example.blending = 0;
             }
+            else if (event.keyboard.keycode == ALLEGRO_KEY_H) {
+               example.hold_bitmap_drawing ^= 1;
+            }
             break;
 
          case ALLEGRO_EVENT_DISPLAY_CLOSE:
             done = true;
             break;
 
+         case ALLEGRO_EVENT_DISPLAY_HALT_DRAWING:
+
+            background = true;
+            al_acknowledge_drawing_halt(event.display.source);
+
+            break;
+         
+         case ALLEGRO_EVENT_DISPLAY_RESUME_DRAWING:
+            background = false;
+            al_acknowledge_drawing_resume(event.display.source);
+            break;
+         
+         case ALLEGRO_EVENT_DISPLAY_RESIZE:
+            al_acknowledge_resize(event.display.source);
+            break;
+              
          case ALLEGRO_EVENT_TIMER:
             update();
             need_redraw = true;
             break;
-        
-         case ALLEGRO_EVENT_MOUSE_BUTTON_DOWN:
-            example.mouse_down = true;
-            example.last_x = event.mouse.x;
-            example.last_y = event.mouse.y;
-            break;
+         
+         case ALLEGRO_EVENT_TOUCH_BEGIN:
+            x = event.touch.x;
+            y = event.touch.y;
+            goto click;
 
-         case ALLEGRO_EVENT_MOUSE_BUTTON_UP: {
+         case ALLEGRO_EVENT_MOUSE_BUTTON_UP:
+            x = event.mouse.x;
+            y = event.mouse.y;
+            goto click;
+            
+         click:
+         {
             int fh = al_get_font_line_height(example.font);
-            example.mouse_down = false;
-            if (event.mouse.x < 40 && event.mouse.y >= h - fh * 5) {
-                int button = (event.mouse.y - (h - fh * 5)) / fh;
-                if (button == 0) {
-                    example.use_memory_bitmaps ^= 1;
-                    change_size(example.bitmap_size);
-                }
-                if (button == 1) {
-                    example.blending++;
-                    if (example.blending == 4)
-                        example.blending = 0;
-                }
-                if (button == 4) {
-                    example.show_help ^= 1;
-                }
+            
+            if (x < fh * 12 && y >= h - fh * 30) {
+               int button = (y - (h - fh * 30)) / (fh * 6);
+               if (button == 0) {
+                  example.use_memory_bitmaps ^= 1;
+                  change_size(example.bitmap_size);
+               }
+               if (button == 1) {
+                  example.blending++;
+                  if (example.blending == 4)
+                     example.blending = 0;
+               }
+               if (button == 3) {
+                  if (x < fh * 6)
+                     remove_sprites(example.sprite_count / 2);
+                  else
+                     add_sprites(example.sprite_count);
+               }
+               if (button == 2) {
+                  int s = example.bitmap_size * 2;
+                  if (x < fh * 6)
+                     s = example.bitmap_size / 2;
+                  change_size(s);
+               }
+               if (button == 4) {
+                  example.show_help ^= 1;
+               }
                 
             }
             break;
          }
-
-         case ALLEGRO_EVENT_MOUSE_AXES:
-            if (example.mouse_down) {
-                double dx = event.mouse.x - example.last_x;
-                double dy = event.mouse.y - example.last_y;
-                if (dy > 4) {
-                    add_sprites(dy / 4);
-                }
-                if (dy < -4) {
-                    remove_sprites(-dy / 4);
-                }
-                if (dx > 4) {
-                    change_size(example.bitmap_size + dx - 4);
-                }
-                if (dx < -4) {
-                    change_size(example.bitmap_size + dx + 4);
-                }
-                
-                example.last_x = event.mouse.x;
-                example.last_y = event.mouse.y;
-            }
-            break;
       }
    }
 
