@@ -29,7 +29,7 @@ static void draw_mouse_button(int but, bool down)
    }
 }
 
-int main(void)
+int main(int argc, char **argv)
 {
    ALLEGRO_DISPLAY *display;
    ALLEGRO_BITMAP *cursor;
@@ -44,12 +44,16 @@ int main(void)
    int mmy = 0;
    int mmz = 0;
    int mmw = 0;
+   int precision = 1;
    bool in = true;
    bool buttons[NUM_BUTTONS] = {false};
    int i;
    float p = 0.0;
    ALLEGRO_COLOR black;
 
+   (void)argc;
+   (void)argv;
+
    if (!al_init()) {
       abort_example("Could not init Allegro.\n");
    }
@@ -59,6 +63,7 @@ int main(void)
    al_install_keyboard();
    al_init_image_addon();
    al_init_font_addon();
+   init_platform_specific();
    
    actual_buttons = al_get_mouse_num_buttons();
    if (actual_buttons > NUM_BUTTONS)
@@ -101,6 +106,7 @@ int main(void)
          al_draw_textf(font, black, 5, 25, 0, "x %i, y %i, z %i, w %i", mx, my, mz, mw);
          al_draw_textf(font, black, 5, 45, 0, "p = %g", p);
          al_draw_textf(font, black, 5, 65, 0, "%s", in ? "in" : "out");
+         al_draw_textf(font, black, 5, 85, 0, "wheel precision (PgUp/PgDn) %d", precision);
          al_set_blender(ALLEGRO_ADD, ALLEGRO_ONE, ALLEGRO_INVERSE_ALPHA);
          mmx = mmy = mmz = 0;
          al_flip_display();
@@ -148,6 +154,19 @@ int main(void)
             }
             break;
 
+         case ALLEGRO_EVENT_KEY_CHAR:
+            if (event.keyboard.keycode == ALLEGRO_KEY_PGUP) {
+               precision++;
+               al_set_mouse_wheel_precision(precision);
+            }
+            else if (event.keyboard.keycode == ALLEGRO_KEY_PGDN) {
+               precision--;
+               if (precision < 1)
+                  precision = 1;
+               al_set_mouse_wheel_precision(precision);
+            }
+            break;
+
          case ALLEGRO_EVENT_DISPLAY_RESIZE:
             al_acknowledge_resize(event.display.source);
             break;
