@@ -21,6 +21,10 @@
 #include "allegro5/internal/aintern_thread.h"
 #include "allegro5/internal/aintern_vector.h"
 
+#ifdef ALLEGRO_ANDROID
+#  include <android/log.h>
+#endif
+
 
 /* tracing */
 typedef struct TRACE_INFO
@@ -53,10 +57,14 @@ static TRACE_INFO trace_info =
    false
 };
 
+static char static_trace_buffer[2048];
+
 /* run-time assertions */
 void (*_al_user_assert_handler)(char const *expr, char const *file,
    int line, char const *func);
 
+void (*_al_user_trace_handler)(char const *message);
+
 
 static void delete_string_list(_AL_VECTOR *v)
 {
@@ -70,20 +78,13 @@ static void delete_string_list(_AL_VECTOR *v)
 }
 
 
-static void configure_logging(void)
+void _al_configure_logging(void)
 {
    ALLEGRO_CONFIG *config;
    char const *v;
    bool got_all = false;
 
-   /* Messages logged before the system driver and allegro5.cfg are
-    * up will always use defaults - but usually nothing is logged
-    * before al_init.
-    */
    config = al_get_system_config();
-   if (!config)
-      return;
-
    v = al_get_config_value(config, "trace", "channels");
    if (v) {
       ALLEGRO_USTR_INFO uinfo;
@@ -122,11 +123,19 @@ static void configure_logging(void)
          delete_string_list(&trace_info.channels);
    }
 
+#ifdef DEBUGMODE
+   trace_info.level = 0;
+#else
+   trace_info.level = 9999;
+#endif
+
    v = al_get_config_value(config, "trace", "level");
    if (v) {
       if (!strcmp(v, "error")) trace_info.level = 3;
       else if (!strcmp(v, "warn")) trace_info.level = 2;
       else if (!strcmp(v, "info")) trace_info.level = 1;
+      else if (!strcmp(v, "debug")) trace_info.level = 0;
+      else if (!strcmp(v, "none")) trace_info.level = 9999;
    }
 
    v = al_get_config_value(config, "trace", "timestamps");
@@ -182,7 +191,19 @@ static void do_trace(const char *msg, ...)
 {
    va_list ap;
 
-   if (trace_info.trace_file) {
+#ifdef ALLEGRO_ANDROID
+   if (true)
+#else
+   if (_al_user_trace_handler)
+#endif
+   {
+      int s = strlen(static_trace_buffer);
+      va_start(ap, msg);
+      vsnprintf(static_trace_buffer + s, sizeof(static_trace_buffer) - s,
+         msg, ap);
+      va_end(ap);
+   }
+   else if (trace_info.trace_file) {
       va_start(ap, msg);
       vfprintf(trace_info.trace_file, msg, ap);
       va_end(ap);
@@ -201,9 +222,8 @@ bool _al_trace_prefix(char const *channel, int level,
    char *name;
    _AL_VECTOR const *v;
 
-   /* XXX logging should be reconfigured if the system driver is reinstalled */
    if (!trace_info.configured) {
-      configure_logging();
+      _al_configure_logging();
    }
 
    if (level < trace_info.level)
@@ -235,7 +255,8 @@ channel_included:
    /* Avoid interleaved output from different threads. */
    _al_mutex_lock(&trace_info.trace_mutex);
 
-   open_trace_file();
+   if (!_al_user_trace_handler)
+      open_trace_file();
 
    do_trace("%-8s ", channel);
    if (level == 0) do_trace("D ");
@@ -243,6 +264,14 @@ channel_included:
    if (level == 2) do_trace("W ");
    if (level == 3) do_trace("E ");
 
+#ifdef ALLEGRO_ANDROID
+   {
+      char pid_buf[16];
+      snprintf(pid_buf, sizeof(pid_buf), "%i: ", gettid());
+      do_trace(pid_buf);
+   }
+#endif
+
 #ifdef ALLEGRO_MSVC
    name = strrchr(file, '\\');
 #else
@@ -255,13 +284,9 @@ channel_included:
       do_trace("%-32s ", function);
    }
    if (trace_info.flags & 4) {
-      double t = al_get_time();
-      /* Kludge:
-       * Very high timers (more than a year?) likely mean the timer
-       * subsystem isn't initialized yet, so print 0.
-       */
-      if (t > 3600 * 24 * 365)
-         t = 0;
+      double t = 0;
+      if (al_is_system_installed())
+         t = al_get_time();
       do_trace("[%10.5f] ", t);
    }
 
@@ -278,7 +303,30 @@ void _al_trace_suffix(const char *msg, ...)
    int olderr = errno;
    va_list ap;
 
-   if (trace_info.trace_file) {
+#ifdef ALLEGRO_ANDROID
+   if (true)
+#else
+   if (_al_user_trace_handler)
+#endif
+   {
+      int s = strlen(static_trace_buffer);
+      va_start(ap, msg);
+      vsnprintf(static_trace_buffer + s, sizeof(static_trace_buffer) - s,
+         msg, ap);
+      va_end(ap);
+
+      if (_al_user_trace_handler) {
+         _al_user_trace_handler(static_trace_buffer);
+      }
+      #ifdef ALLEGRO_ANDROID
+      else {
+         (void)__android_log_print(ANDROID_LOG_INFO, "allegro",
+            static_trace_buffer);
+      }
+      #endif
+      static_trace_buffer[0] = '\0';
+   }
+   else if (trace_info.trace_file) {
       va_start(ap, msg);
       vfprintf(trace_info.trace_file, msg, ap);
       va_end(ap);
@@ -320,4 +368,12 @@ void al_register_assert_handler(void (*handler)(char const *expr,
 }
 
 
+/* Function: al_register_trace_handler
+ */
+void al_register_trace_handler(void (*handler)(char const *))
+{
+   _al_user_trace_handler = handler;
+}
+
+
 /* vim: set sts=3 sw=3 et: */
