@@ -68,6 +68,7 @@ ALLEGRO_DISPLAY *al_create_display(int w, int h)
    display->min_h = 0;
    display->max_w = 0;
    display->max_h = 0;
+   display->use_constraints = false;
 
    display->vertex_cache = 0;
    display->num_cache_vertices = 0;
@@ -121,10 +122,6 @@ ALLEGRO_DISPLAY *al_create_display(int w, int h)
 #endif
    }
 
-   // TODO: Remove this once all the implementations set the title correctly themselves.
-   // Issue #544.
-   al_set_window_title(display, al_get_new_window_title());
-   
    if (settings->settings[ALLEGRO_AUTO_CONVERT_BITMAPS]) {
       /* We convert video bitmaps to memory bitmaps when the display is
        * destroyed, so seems only fair to re-convertt hem when the
@@ -628,4 +625,31 @@ void al_set_render_state(ALLEGRO_RENDER_STATE state, int value)
    }
 }
 
+/* Function: al_backup_dirty_bitmaps
+ */
+void al_backup_dirty_bitmaps(ALLEGRO_DISPLAY *display)
+{
+   unsigned int i;
+
+   for (i = 0; i < display->bitmaps._size; i++) {
+      ALLEGRO_BITMAP **bptr = (ALLEGRO_BITMAP **)_al_vector_ref(&display->bitmaps, i);
+      ALLEGRO_BITMAP *bmp = *bptr;
+      if (_al_get_bitmap_display(bmp) == display) {
+         if (bmp->vt && bmp->vt->backup_dirty_bitmap) {
+            bmp->vt->backup_dirty_bitmap(bmp);
+	 }
+      }
+   }
+}
+
+/* Function: al_apply_window_constraints
+ */
+void al_apply_window_constraints(ALLEGRO_DISPLAY *display, bool onoff)
+{
+   display->use_constraints = onoff;
+
+   if (display->vt && display->vt->apply_window_constraints)
+      display->vt->apply_window_constraints(display, onoff);
+}
+
 /* vim: set sts=3 sw=3 et: */
