@@ -228,6 +228,9 @@ void _al_osx_keyboard_was_installed(BOOL install) {
 -(void) exitFullScreenWindowMode;
 -(void) finishExitingFullScreenWindowMode;
 -(void) maximize;
+-(NSRect) windowWillUseStandardFrame:
+   (NSWindow *) window
+   defaultFrame: (NSRect) newFrame;
 @end
 
 @implementation ALWindow
@@ -574,6 +577,18 @@ void _al_osx_mouse_was_installed(BOOL install) {
 #endif
    ALLEGRO_EVENT_SOURCE *es = &dpy->parent.es;
 
+   /* Restore max. constraints when the window has been un-maximized.
+    * Note: isZoomed will return false in FullScreen mode.
+    */
+   if (dpy_ptr->use_constraints &&
+      !(dpy_ptr->flags & ALLEGRO_FULLSCREEN_WINDOW) && ![window isZoomed])
+   {
+      NSSize max_size;
+      max_size.width = (dpy_ptr->max_w > 0) ? dpy_ptr->max_w : FLT_MAX;
+      max_size.height = (dpy_ptr->max_h > 0) ? dpy_ptr->max_h : FLT_MAX;
+      [window setContentMaxSize: max_size];
+   }
+
    _al_event_source_lock(es);
    if (_al_event_source_needs_to_generate_event(es)) {
       ALLEGRO_EVENT event;
@@ -591,14 +606,14 @@ void _al_osx_mouse_was_installed(BOOL install) {
 {
     (void)notification;
     ALLEGRO_DISPLAY *display = dpy_ptr;
-    display->flags |= ALLEGRO_MAXIMIZED;
+    display->flags |= ALLEGRO_FULLSCREEN_WINDOW;
 }
 
 -(void)windowWillExitFullScreen:(NSNotification *)notification
 {
     (void)notification;
     ALLEGRO_DISPLAY *display = dpy_ptr;
-    display->flags &= ~ALLEGRO_MAXIMIZED;
+    display->flags &= ~ALLEGRO_FULLSCREEN_WINDOW;
 }
 
 -(void) enterFullScreenWindowMode
@@ -623,6 +638,35 @@ void _al_osx_mouse_was_installed(BOOL install) {
    [dpy->win performZoom: nil];
 }
 
+/* Called by NSWindow's zoom: method while determining the frame
+ * a window may be zoomed to.
+ * We use it to update max. constraints values when the window changes
+ * its maximized state.
+ */
+-(NSRect) windowWillUseStandardFrame:
+   (NSWindow *) window
+   defaultFrame: (NSRect) newFrame
+{
+   NSSize max_size;
+
+   if (dpy_ptr->use_constraints) {
+      if (dpy_ptr->flags & ALLEGRO_MAXIMIZED) {
+         max_size.width = FLT_MAX;
+         max_size.height = FLT_MAX;
+         newFrame.size.width = max_size.width;
+         newFrame.size.height = max_size.height;
+      }
+      else {
+         max_size.width = (dpy_ptr->max_w > 0) ? dpy_ptr->max_w : FLT_MAX;
+         max_size.height = (dpy_ptr->max_h > 0) ? dpy_ptr->max_h : FLT_MAX;
+      }
+
+      [window setContentMaxSize: max_size];
+   }
+
+   return newFrame;
+}
+
 -(void) exitFullScreenWindowMode
 {
    ALLEGRO_DISPLAY_OSX_WIN *dpy =  (ALLEGRO_DISPLAY_OSX_WIN*) dpy_ptr;
@@ -1010,6 +1054,14 @@ static void osx_get_opengl_pixelformat_attributes(ALLEGRO_DISPLAY_OSX_WIN *dpy)
    [win setMinSize: NSMakeSize(MINIMUM_WIDTH / screen_scale_factor,
                                MINIMUM_HEIGHT / screen_scale_factor)];
 
+   /* Maximize the window and update its width & height information */
+   if (dpy->parent.flags & ALLEGRO_MAXIMIZED) {
+      [win setFrame: [screen visibleFrame] display: true animate: false];
+      NSRect content = [win contentRectForFrameRect: [win frame]];
+      dpy->parent.w = content.size.width;
+      dpy->parent.h = content.size.height;
+   }
+
    /* Place the window, respecting the location set by the user with
     * al_set_new_window_position().
     * If the user never called al_set_new_window_position, we simply let
@@ -1888,20 +1940,19 @@ static bool acknowledge_resize_display_win(ALLEGRO_DISPLAY *d)
    content = [window convertRectToBacking: content];
 #endif
 
-   int ow = d->w, oh = d->h;
+   /* At any moment when a window has been changed its size we either
+    * clear ALLEGRO_MAXIMIZED flag (e.g. resize was done by a human)
+    * or restore the flag back (at the end of live resize caused by zoom).
+    * Note: affects zoom:(id)sender (if you will debug it).
+    */
+   if (!(d->flags & ALLEGRO_FULLSCREEN_WINDOW) && ![window isZoomed])
+      d->flags &= ~ALLEGRO_MAXIMIZED;
+   else if (!(d->flags & ALLEGRO_MAXIMIZED))
+      d->flags |= ALLEGRO_MAXIMIZED;
+
    d->w = NSWidth(content);
    d->h = NSHeight(content);
 
-   if (d->w < ow || d->h < oh) {
-      /* In OSX there is no special "maximized" state, nor is there a specific maximized size.
-       * Instead each application can override the size it prefers as maximized size. Therefore
-       * you can maximize a window, and then resize it and it will still be maximized. Because
-       * determining whether a window is maximized is hard this heuristic works most of the
-       * time (except when you shrink a window and it still is maximized afterwards, which is
-       * very well possible).
-       */
-      d->flags &= ~ALLEGRO_MAXIMIZED;
-   }
    if (d->ogl_extras->backbuffer) {
       _al_ogl_resize_backbuffer(d->ogl_extras->backbuffer, d->w, d->h);
    }
@@ -1932,19 +1983,30 @@ static bool resize_display_win(ALLEGRO_DISPLAY *d, int w, int h)
    w = _ALLEGRO_MAX(w, MINIMUM_WIDTH / scale_factor);
    h = _ALLEGRO_MAX(h, MINIMUM_HEIGHT / scale_factor);
 
-   if (d->min_w > 0 && w < d->min_w) {
-      w = d->min_w;
-   }
-   if (d->min_h > 0 && h < d->min_h) {
-      h = d->min_h;
-   }
-   if (d->max_w > 0 && w > d->max_w) {
-      w = d->max_w;
-   }
-   if (d->max_h > 0 && h > d->max_h) {
-      h = d->max_h;
+   if (d->use_constraints) {
+      if (d->min_w > 0 && w < d->min_w) {
+         w = d->min_w;
+      }
+      if (d->min_h > 0 && h < d->min_h) {
+         h = d->min_h;
+      }
+      /* Don't use max. constraints when a window is maximized. */
+      if (!(d->flags & ALLEGRO_MAXIMIZED)) {
+         if (d->max_w > 0 && w > d->max_w) {
+            w = d->max_w;
+         }
+         if (d->max_h > 0 && h > d->max_h) {
+            h = d->max_h;
+         }
+      }
    }
 
+   /* Set new width & height values to content rectangle
+    * before calling 'set_frame' below.
+    */
+   content.size.width = w;
+   content.size.height = h;
+
    NSRect rc = [window frameRectForContentRect: content];
    rc.origin = current.origin;
 
@@ -2086,15 +2148,6 @@ static void get_window_position(ALLEGRO_DISPLAY* display, int* px, int* py)
 static bool set_window_constraints(ALLEGRO_DISPLAY* display,
    int min_w, int min_h, int max_w, int max_h)
 {
-   ALLEGRO_DISPLAY_OSX_WIN* d = (ALLEGRO_DISPLAY_OSX_WIN*) display;
-   NSWindow* window = d->win;
-   float scale_factor = 1.0;
-#if MAC_OS_X_VERSION_MIN_REQUIRED >= 1070
-   if ([window respondsToSelector:@selector(backingScaleFactor)]) {
-      scale_factor = [window backingScaleFactor];
-   }
-#endif
-
    if (min_w > 0 && min_w < MINIMUM_WIDTH) {
       min_w = MINIMUM_WIDTH;
    }
@@ -2107,40 +2160,7 @@ static bool set_window_constraints(ALLEGRO_DISPLAY* display,
    display->max_w = max_w;
    display->max_h = max_h;
 
-   NSSize min_size = [window contentMinSize];
-   NSSize max_size = [window contentMaxSize];
-
-   if (display->min_w > 0) {
-      min_size.width = display->min_w / scale_factor;
-   }
-   else {
-      min_size.width = MINIMUM_WIDTH / scale_factor;
-   }
-   if (display->min_h > 0) {
-      min_size.height = display->min_h / scale_factor;
-   }
-   else {
-      min_size.height = MINIMUM_HEIGHT / scale_factor;
-   }
-   if (display->max_w > 0) {
-      max_size.width = display->max_w / scale_factor;
-   }
-   else {
-      max_size.width = FLT_MAX;
-   }
-   if (display->max_h > 0) {
-      max_size.height = display->max_h / scale_factor;
-   }
-   else {
-     max_size.height = FLT_MAX;
-   }
-
-  [window setContentMaxSize:max_size];
-  [window setContentMinSize:min_size];
-
-  al_resize_display(display, display->w, display->h);
-
-  return true;
+   return true;
 }
 
 static bool get_window_constraints(ALLEGRO_DISPLAY* display,
@@ -2154,6 +2174,47 @@ static bool get_window_constraints(ALLEGRO_DISPLAY* display,
    return true;
 }
 
+static void apply_window_constraints(ALLEGRO_DISPLAY *display,
+   bool onoff)
+{
+   ALLEGRO_DISPLAY_OSX_WIN* d = (ALLEGRO_DISPLAY_OSX_WIN*) display;
+   NSWindow* window = d->win;
+   float scale_factor = 1.0;
+   NSSize max_size;
+   NSSize min_size;
+#if MAC_OS_X_VERSION_MIN_REQUIRED >= 1070
+   if ([window respondsToSelector:@selector(backingScaleFactor)]) {
+      scale_factor = [window backingScaleFactor];
+   }
+#endif
+
+   if (onoff) {
+      min_size.width = display->min_w / scale_factor;
+      min_size.height = display->min_h / scale_factor;
+
+      if (display->max_w > 0)
+         max_size.width = display->max_w / scale_factor;
+      else
+         max_size.width = FLT_MAX;
+
+      if (display->max_h > 0)
+         max_size.height = display->max_h / scale_factor;
+      else
+         max_size.height = FLT_MAX;
+
+      al_resize_display(display, display->w, display->h);
+   }
+   else {
+      min_size.width = MINIMUM_WIDTH / scale_factor;
+      min_size.height = MINIMUM_HEIGHT / scale_factor;
+      max_size.width = FLT_MAX;
+      max_size.height = FLT_MAX;
+   }
+
+   [window setContentMaxSize:max_size];
+   [window setContentMinSize:min_size];
+}
+
 /* set_window_title:
  * Set the title of the window with this display
  */
@@ -2310,6 +2371,7 @@ ALLEGRO_DISPLAY_INTERFACE* _al_osx_get_display_driver_win(void)
       vt->set_window_position = set_window_position;
       vt->get_window_constraints = get_window_constraints;
       vt->set_window_constraints = set_window_constraints;
+      vt->apply_window_constraints = apply_window_constraints;
       vt->set_window_title = set_window_title;
       vt->set_display_flag = set_display_flag;
       vt->set_icons = set_icons;
