@@ -162,7 +162,7 @@ void _al_osx_mouse_generate_event(NSEvent* evt, ALLEGRO_DISPLAY* dpy)
       // Y-coordinates in OS X start from the bottom.
       pos.y = NSHeight(frm) - pos.y;
 #if MAC_OS_X_VERSION_MIN_REQUIRED >= 1070
-      scaling_factor = [[[evt window] contentView] backingScaleFactor];
+      scaling_factor = [[evt window] backingScaleFactor];
 #endif
    }
    else
@@ -174,8 +174,10 @@ void _al_osx_mouse_generate_event(NSEvent* evt, ALLEGRO_DISPLAY* dpy)
    }
    dx *= scaling_factor;
    dy *= scaling_factor;
-   if (osx_mouse.warped) {
-      osx_mouse.warped = FALSE;
+   if (osx_mouse.warped && type == ALLEGRO_EVENT_MOUSE_AXES) {
+       dx -= osx_mouse.warped_x;
+       dy -= osx_mouse.warped_y;
+       osx_mouse.warped = FALSE;
    }
    _al_event_source_lock(&osx_mouse.parent.es);
    if ((within || b_change || type == ALLEGRO_EVENT_MOUSE_LEAVE_DISPLAY)
@@ -187,7 +189,7 @@ void _al_osx_mouse_generate_event(NSEvent* evt, ALLEGRO_DISPLAY* dpy)
       // Note: we use 'allegro time' rather than the time stamp
       // from the event
       mouse_event->timestamp = al_get_time();
-                mouse_event->display = dpy;
+      mouse_event->display = dpy;
       mouse_event->button = b;
       mouse_event->x = pos.x * scaling_factor;
       mouse_event->y = pos.y * scaling_factor;
@@ -340,7 +342,7 @@ static bool osx_set_mouse_xy(ALLEGRO_DISPLAY *dpy_, int x, int y)
 
       CGPoint point_pos = CGPointMake(x, y);
 #if MAC_OS_X_VERSION_MIN_REQUIRED >= 1070
-      point_pos = [[window contentView] convertPointFromBacking: point_pos];
+      point_pos = NSPointToCGPoint([[window contentView] convertPointFromBacking: NSPointFromCGPoint(point_pos)]);
 #endif
       pos.x = content.origin.x + point_pos.x;
       pos.y = rect.size.height - content.origin.y - content.size.height + point_pos.y;
