@@ -76,6 +76,15 @@ void _al_ogl_set_target_bitmap(ALLEGRO_DISPLAY *display, ALLEGRO_BITMAP *bitmap)
 }
 
 
+void _al_ogl_unset_target_bitmap(ALLEGRO_DISPLAY *display,
+   ALLEGRO_BITMAP *target)
+{
+   if (!target)
+      return;
+   _al_ogl_finalize_fbo(display, target);
+}
+
+
 /* Function: al_set_current_opengl_context
  */
 void al_set_current_opengl_context(ALLEGRO_DISPLAY *display)
@@ -176,16 +185,7 @@ bool _al_ogl_resize_backbuffer(ALLEGRO_BITMAP *b, int w, int h)
    extra->true_w = w;
    extra->true_h = h;
 
-   if (!IS_IPHONE) {
-      b->memory = NULL;
-   }
-   else {
-      /* iPhone port still expects the buffer to be present. */
-      /* FIXME: lazily manage memory */
-      size_t bytes = pitch * h;
-      al_free(b->memory);
-      b->memory = al_calloc(1, bytes);
-   }
+   b->memory = NULL;
 
    return true;
 }
