@@ -45,6 +45,7 @@ struct ALLEGRO_TIMER
    double speed_secs;
    int64_t count;
    double counter;		/* counts down to zero=blastoff */
+   _AL_LIST_ITEM *dtor_item;
 };
 
 
@@ -53,10 +54,11 @@ struct ALLEGRO_TIMER
  * The timer thread that runs in the background to drive the timers.
  */
 
-static _AL_MUTEX timers_mutex = _AL_MUTEX_UNINITED;
+static ALLEGRO_MUTEX *timers_mutex;
 static _AL_VECTOR active_timers = _AL_VECTOR_INITIALIZER(ALLEGRO_TIMER *);
 static _AL_THREAD * volatile timer_thread = NULL;
-
+static ALLEGRO_COND *timer_cond = NULL;
+static bool destroy_thread = false;
 
 
 /* timer_thread_proc: [timer thread]
@@ -99,9 +101,16 @@ static void timer_thread_proc(_AL_THREAD *self, void *unused)
    double interval = 0.032768;
 
    while (!_al_get_thread_should_stop(self)) {
+      al_lock_mutex(timers_mutex);
+      while (_al_vector_size(&active_timers) == 0 && !destroy_thread) {
+         al_wait_cond(timer_cond, timers_mutex);
+         old_time = al_get_time() - interval;
+      }
+      al_unlock_mutex(timers_mutex);
+
       al_rest(interval);
 
-      _al_mutex_lock(&timers_mutex);
+      al_lock_mutex(timers_mutex);
       {
          /* Calculate actual time elapsed.  */
          new_time = al_get_time();
@@ -111,7 +120,7 @@ static void timer_thread_proc(_AL_THREAD *self, void *unused)
          /* Handle a tick.  */
          interval = timer_thread_handle_tick(interval);
       }
-      _al_mutex_unlock(&timers_mutex);
+      al_unlock_mutex(timers_mutex);
    }
 
    (void)unused;
@@ -152,9 +161,24 @@ static double timer_thread_handle_tick(double interval)
 static void shutdown_timers(void)
 {
    ASSERT(_al_vector_size(&active_timers) == 0);
-   ASSERT(timer_thread == NULL);
 
-   _al_mutex_destroy(&timers_mutex);
+   _al_vector_free(&active_timers);
+
+   if (timer_thread != NULL) {
+      destroy_thread = true;
+      al_lock_mutex(timers_mutex);
+      al_signal_cond(timer_cond);
+      al_unlock_mutex(timers_mutex);
+      _al_thread_join(timer_thread);
+   }
+
+   al_free(timer_thread);
+
+   timer_thread = NULL;
+
+   al_destroy_mutex(timers_mutex);
+
+   al_destroy_cond(timer_cond);
 }
 
 
@@ -166,12 +190,10 @@ static void enable_timer(ALLEGRO_TIMER *timer, bool reset_counter)
 {
    ASSERT(timer);
    {
-      size_t new_size;
-
       if (timer->started)
          return;
 
-      _al_mutex_lock(&timers_mutex);
+      al_lock_mutex(timers_mutex);
       {
          ALLEGRO_TIMER **slot;
 
@@ -183,11 +205,12 @@ static void enable_timer(ALLEGRO_TIMER *timer, bool reset_counter)
          slot = _al_vector_alloc_back(&active_timers);
          *slot = timer;
 
-         new_size = _al_vector_size(&active_timers);
+         al_signal_cond(timer_cond);
       }
-      _al_mutex_unlock(&timers_mutex);
+      al_unlock_mutex(timers_mutex);
 
-      if (new_size == 1) {
+      if (timer_thread == NULL) {
+         destroy_thread = false;
          timer_thread = al_malloc(sizeof(_AL_THREAD));
          _al_thread_create(timer_thread, timer_thread_proc, NULL);
       }
@@ -198,7 +221,8 @@ static void enable_timer(ALLEGRO_TIMER *timer, bool reset_counter)
 
 void _al_init_timers(void)
 {
-   _al_mutex_init(&timers_mutex);
+   timers_mutex = al_create_mutex();
+   timer_cond = al_create_cond();
    _al_add_exit_func(shutdown_timers, "shutdown_timers");
 }
 
@@ -232,7 +256,7 @@ ALLEGRO_TIMER *al_create_timer(double speed_secs)
          timer->speed_secs = speed_secs;
          timer->counter = 0;
 
-         _al_register_destructor(_al_dtor_list, "timer", timer,
+         timer->dtor_item = _al_register_destructor(_al_dtor_list, "timer", timer,
             (void (*)(void *)) al_destroy_timer);
       }
 
@@ -249,7 +273,7 @@ void al_destroy_timer(ALLEGRO_TIMER *timer)
    if (timer) {
       al_stop_timer(timer);
 
-      _al_unregister_destructor(_al_dtor_list, timer);
+      _al_unregister_destructor(_al_dtor_list, timer->dtor_item);
 
       _al_event_source_free(&timer->es);
       al_free(timer);
@@ -282,28 +306,15 @@ void al_stop_timer(ALLEGRO_TIMER *timer)
 {
    ASSERT(timer);
    {
-      _AL_THREAD *thread_to_join = NULL;
-
       if (!timer->started)
          return;
 
-      _al_mutex_lock(&timers_mutex);
+      al_lock_mutex(timers_mutex);
       {
          _al_vector_find_and_delete(&active_timers, &timer);
          timer->started = false;
-
-         if (_al_vector_size(&active_timers) == 0) {
-            _al_vector_free(&active_timers);
-            thread_to_join = timer_thread;
-            timer_thread = NULL;
-         }
-      }
-      _al_mutex_unlock(&timers_mutex);
-
-      if (thread_to_join) {
-         _al_thread_join(thread_to_join);
-         al_free(thread_to_join);
       }
+      al_unlock_mutex(timers_mutex);
    }
 }
 
@@ -338,7 +349,7 @@ void al_set_timer_speed(ALLEGRO_TIMER *timer, double new_speed_secs)
    ASSERT(timer);
    ASSERT(new_speed_secs > 0);
 
-   _al_mutex_lock(&timers_mutex);
+   al_lock_mutex(timers_mutex);
    {
       if (timer->started) {
          timer->counter -= timer->speed_secs;
@@ -347,7 +358,7 @@ void al_set_timer_speed(ALLEGRO_TIMER *timer, double new_speed_secs)
 
       timer->speed_secs = new_speed_secs;
    }
-   _al_mutex_unlock(&timers_mutex);
+   al_unlock_mutex(timers_mutex);
 }
 
 
@@ -369,11 +380,11 @@ void al_set_timer_count(ALLEGRO_TIMER *timer, int64_t new_count)
 {
    ASSERT(timer);
 
-   _al_mutex_lock(&timers_mutex);
+   al_lock_mutex(timers_mutex);
    {
       timer->count = new_count;
    }
-   _al_mutex_unlock(&timers_mutex);
+   al_unlock_mutex(timers_mutex);
 }
 
 
@@ -384,15 +395,14 @@ void al_add_timer_count(ALLEGRO_TIMER *timer, int64_t diff)
 {
    ASSERT(timer);
 
-   _al_mutex_lock(&timers_mutex);
+   al_lock_mutex(timers_mutex);
    {
       timer->count += diff;
    }
-   _al_mutex_unlock(&timers_mutex);
+   al_unlock_mutex(timers_mutex);
 }
 
 
-
 /* timer_handle_tick: [timer thread]
  *  Handle a single tick.
  */
