@@ -1,6 +1,6 @@
-/*         ______   ___    ___ 
+/*         ______   ___    ___
  *        /\  _  \ /\_ \  /\_ \
- *        \ \ \L\ \\//\ \ \//\ \      __     __   _ __   ___ 
+ *        \ \ \L\ \\//\ \ \//\ \      __     __   _ __   ___
  *         \ \  __ \ \ \ \  \ \ \   /'__`\ /'_ `\/\`'__\/ __`\
  *          \ \ \/\ \ \_\ \_ \_\ \_/\  __//\ \L\ \ \ \//\ \L\ \
  *           \ \_\ \_\/\____\/\____\ \____\ \____ \ \_\\ \____/
@@ -22,6 +22,9 @@
 #define WINVER 0x0501
 #endif
 #endif
+
+#define UNICODE
+
 #include <windows.h>
 
 #include "allegro5/allegro.h"
@@ -31,6 +34,7 @@
 #include "allegro5/internal/aintern_bitmap.h"
 #include "allegro5/internal/aintern_opengl.h"
 #include "allegro5/internal/aintern_vector.h"
+#include "allegro5/internal/aintern_wclipboard.h"
 #include "allegro5/platform/aintwin.h"
 
 #include "wgl.h"
@@ -59,6 +63,8 @@ typedef struct WGL_DISPLAY_PARAMETERS {
    volatile bool init_failed;
    HANDLE AckEvent;
    int window_x, window_y;
+   /* Not owned. */
+   const char* window_title;
 } WGL_DISPLAY_PARAMETERS;
 
 
@@ -88,19 +94,6 @@ static char* get_error_desc(DWORD err)
    return err_msg;
 }
 
-/* Helper to set up GL state as we want it. */
-static void setup_gl(ALLEGRO_DISPLAY *d)
-{
-   glViewport(0, 0, d->w, d->h);
-
-   glMatrixMode(GL_PROJECTION);
-   glLoadIdentity();
-   glOrtho(0, d->w, d->h, 0, -1, 1);
-
-   glMatrixMode(GL_MODELVIEW);
-   glLoadIdentity();
-}
-
 static bool is_wgl_extension_supported(const char *extension, HDC dc)
 {
    _ALLEGRO_wglGetExtensionsStringARB_t _wglGetExtensionsStringARB;
@@ -181,7 +174,7 @@ static bool init_pixel_format_extensions(void)
     */
    _wglGetPixelFormatAttribivARB =
       (_ALLEGRO_wglGetPixelFormatAttribivARB_t)wglGetProcAddress("wglGetPixelFormatAttribivARB");
-   _wglGetPixelFormatAttribivEXT = 
+   _wglGetPixelFormatAttribivEXT =
       (_ALLEGRO_wglGetPixelFormatAttribivEXT_t)wglGetProcAddress("wglGetPixelFormatAttribivEXT");
 
    if (!_wglGetPixelFormatAttribivARB && !_wglGetPixelFormatAttribivEXT) {
@@ -348,11 +341,11 @@ static bool decode_pixel_format_attrib(ALLEGRO_EXTRA_DISPLAY_SETTINGS *eds, int
 
    for (i = 0; i < num_attribs; i++) {
       /* Not interested if it doesn't support OpenGL or window drawing or RGBA. */
-      if (attrib[i] == WGL_SUPPORT_OPENGL_ARB && value[i] == 0) {	
+      if (attrib[i] == WGL_SUPPORT_OPENGL_ARB && value[i] == 0) {
          ALLEGRO_INFO("OpenGL Unsupported\n");
          return false;
       }
-      else if (attrib[i] == WGL_DRAW_TO_WINDOW_ARB && value[i] == 0) {	
+      else if (attrib[i] == WGL_DRAW_TO_WINDOW_ARB && value[i] == 0) {
          ALLEGRO_INFO("Can't draw to window\n");
          return false;
       }
@@ -400,7 +393,7 @@ static bool decode_pixel_format_attrib(ALLEGRO_EXTRA_DISPLAY_SETTINGS *eds, int
             eds->settings[ALLEGRO_SWAP_METHOD] = 0;
          else if (value[i] == WGL_SWAP_COPY_ARB)
             eds->settings[ALLEGRO_SWAP_METHOD] = 1;
-         else if (value[i] == WGL_SWAP_EXCHANGE_ARB) 
+         else if (value[i] == WGL_SWAP_EXCHANGE_ARB)
             eds->settings[ALLEGRO_SWAP_METHOD] = 2;
       }
 
@@ -512,7 +505,7 @@ static ALLEGRO_EXTRA_DISPLAY_SETTINGS* read_pixel_format_ext(int fmt, HDC dc)
 
       /* The following are used by extensions that add to WGL_pixel_format.
        * If WGL_p_f isn't supported though, we can't use the (then invalid)
-       * enums. We can't use any magic number either, so we settle for 
+       * enums. We can't use any magic number either, so we settle for
        * replicating one. The pixel format decoder
        * (decode_pixel_format_attrib()) doesn't care about duplicates.
        */
@@ -547,7 +540,7 @@ static ALLEGRO_EXTRA_DISPLAY_SETTINGS* read_pixel_format_ext(int fmt, HDC dc)
    else {
       ret = 0;
    }
-   
+
    if (!ret) {
       ALLEGRO_ERROR("wglGetPixelFormatAttrib failed! %s\n",
                      get_error_desc(GetLastError()));
@@ -562,7 +555,7 @@ static ALLEGRO_EXTRA_DISPLAY_SETTINGS* read_pixel_format_ext(int fmt, HDC dc)
    }
 
    al_free(value);
-   
+
    /* Hack: for some reason this happens for me under Wine. */
    if (eds &&
       eds->settings[ALLEGRO_RED_SHIFT] == 0 &&
@@ -584,7 +577,7 @@ static bool change_display_mode(ALLEGRO_DISPLAY *d)
    DEVMODE dm;
    DEVMODE fallback_dm;
    DISPLAY_DEVICE dd;
-   char* dev_name = NULL;
+   TCHAR* dev_name = NULL;
    int i, modeswitch, result;
    int fallback_dm_valid = 0;
    int bpp;
@@ -858,23 +851,21 @@ static ALLEGRO_EXTRA_DISPLAY_SETTINGS** get_available_pixel_formats_old(int *cou
 static bool select_pixel_format(ALLEGRO_DISPLAY_WGL *d, HDC dc)
 {
    ALLEGRO_EXTRA_DISPLAY_SETTINGS **eds = NULL;
-   ALLEGRO_SYSTEM *system = (void *)al_get_system_driver();
+   ALLEGRO_CONFIG *sys_cfg = al_get_system_config();
    int eds_count = 0;
    int i;
    bool force_old = false;
+   const char *selection_mode;
 
-   if (system->config) {
-      const char *selection_mode;
-      selection_mode = al_get_config_value(system->config, "graphics",
-                          "config_selection");
-      if (selection_mode && selection_mode[0] != '\0') {
-         if (!_al_stricmp(selection_mode, "old")) {
-            ALLEGRO_INFO("Forcing OLD visual selection method.\n");
-            force_old = true;
-         }
-         else if (!_al_stricmp(selection_mode, "new"))
-            force_old = false;
+   selection_mode = al_get_config_value(sys_cfg, "graphics",
+                       "config_selection");
+   if (selection_mode && selection_mode[0] != '\0') {
+      if (!_al_stricmp(selection_mode, "old")) {
+         ALLEGRO_INFO("Forcing OLD visual selection method.\n");
+         force_old = true;
       }
+      else if (!_al_stricmp(selection_mode, "new"))
+         force_old = false;
    }
 
    if (!force_old)
@@ -927,6 +918,7 @@ static bool create_display_internals(ALLEGRO_DISPLAY_WGL *wgl_disp)
    ALLEGRO_DISPLAY_WIN *win_disp = (void*)wgl_disp;
    WGL_DISPLAY_PARAMETERS ndp;
    int window_x, window_y;
+   int major, minor;
 
    /* The window is created in a separate thread so we need to pass this
     * TLS on
@@ -934,6 +926,7 @@ static bool create_display_internals(ALLEGRO_DISPLAY_WGL *wgl_disp)
    al_get_new_window_position(&window_x, &window_y);
    ndp.window_x = window_x;
    ndp.window_y = window_y;
+   ndp.window_title = al_get_new_window_title();
 
    /* _beginthread closes the handle automatically. */
    ndp.display = wgl_disp;
@@ -960,15 +953,23 @@ static bool create_display_internals(ALLEGRO_DISPLAY_WGL *wgl_disp)
 
    /* WGL display lists cannot be shared with the API currently in use. */
    disp->ogl_extras->is_shared = false;
-   
+
    if (!select_pixel_format(wgl_disp, wgl_disp->dc)) {
       destroy_display_internals(wgl_disp);
       return false;
    }
 
-   if (disp->flags & ALLEGRO_OPENGL_3_0) {
+   major = _al_get_suggested_display_option(disp,
+      ALLEGRO_OPENGL_MAJOR_VERSION, 0);
+   minor = _al_get_suggested_display_option(disp,
+      ALLEGRO_OPENGL_MINOR_VERSION, 0);
+
+   if ((disp->flags & ALLEGRO_OPENGL_3_0) || major != 0) {
+      if (major == 0)
+         major = 3;
       bool fc = (disp->flags & ALLEGRO_OPENGL_FORWARD_COMPATIBLE) != 0;
-      wgl_disp->glrc = init_ogl_context_ex(wgl_disp->dc, fc, 3, 0);
+      wgl_disp->glrc = init_ogl_context_ex(wgl_disp->dc, fc, major,
+         minor);
    }
    else {
       wgl_disp->glrc = wglCreateContext(wgl_disp->dc);
@@ -1002,15 +1003,6 @@ static bool create_display_internals(ALLEGRO_DISPLAY_WGL *wgl_disp)
       disp->extra_settings.settings[ALLEGRO_COMPATIBLE_DISPLAY] = 0;
    }
 
-   disp->ogl_extras->backbuffer = _al_ogl_create_backbuffer(disp);
-   if (!disp->ogl_extras->backbuffer) {
-      ALLEGRO_ERROR("Failed to create a backbuffer.\n");
-      destroy_display_internals(wgl_disp);
-      return false;
-   }
-
-   al_identity_transform(&al_get_backbuffer(disp)->transform);
-
    /* Try to enable or disable vsync as requested */
    /* NOTE: my drivers claim I don't have WGL_EXT_swap_control
     * (according to al_have_opengl_extension), but wglSwapIntervalEXT
@@ -1024,7 +1016,7 @@ static bool create_display_internals(ALLEGRO_DISPLAY_WGL *wgl_disp)
          wglSwapIntervalEXT(0);
       }
    }
- 
+
    win_disp->mouse_selected_hcursor = 0;
    win_disp->mouse_cursor_shown = false;
    win_disp->can_acknowledge = false;
@@ -1032,7 +1024,7 @@ static bool create_display_internals(ALLEGRO_DISPLAY_WGL *wgl_disp)
    _al_win_grab_input(win_disp);
 
    if (disp->extra_settings.settings[ALLEGRO_COMPATIBLE_DISPLAY])
-      setup_gl(disp);
+      _al_ogl_setup_gl(disp);
 
    return true;
 }
@@ -1078,6 +1070,8 @@ static ALLEGRO_DISPLAY* wgl_create_display(int w, int h)
    _al_win_set_system_mouse_cursor(display, ALLEGRO_SYSTEM_MOUSE_CURSOR_ARROW);
    _al_win_show_mouse_cursor(display);
 
+   _al_win_post_create_window(display);
+
    return display;
 }
 
@@ -1138,11 +1132,14 @@ static void wgl_destroy_display(ALLEGRO_DISPLAY *disp)
    if (system->mouse_grab_display == disp)
       system->mouse_grab_display = NULL;
 
+   _al_win_destroy_display_icons(disp);
+
    destroy_display_internals(wgl_disp);
    _al_event_source_free(&disp->es);
    _al_vector_find_and_delete(&system->system.displays, &disp);
 
    _al_vector_free(&disp->bitmaps);
+   _al_vector_free(&((ALLEGRO_DISPLAY_WIN*) disp)->msg_callbacks);
    al_free(disp->ogl_extras);
 
    if (old_disp != disp)
@@ -1171,6 +1168,8 @@ static bool wgl_set_current_display(ALLEGRO_DISPLAY *d)
       _al_ogl_set_extensions(d->ogl_extras->extension_api);
    }
 
+   _al_ogl_update_render_state(d);
+
    return true;
 }
 
@@ -1198,11 +1197,11 @@ static void display_thread_proc(void *arg)
    ALLEGRO_DISPLAY_WIN *win_disp = (void*)disp;
    MSG msg;
 
-   al_set_new_window_position(ndp->window_x, ndp->window_y);
-
    /* So that we can call the functions using TLS from this thread. */
    al_set_new_display_flags(disp->flags);
-   
+   al_set_new_window_position(ndp->window_x, ndp->window_y);
+   al_set_new_window_title(ndp->window_title);
+
    if (disp->flags & ALLEGRO_FULLSCREEN) {
       if (!change_display_mode(disp)) {
          win_disp->thread_ended = true;
@@ -1246,7 +1245,7 @@ static void display_thread_proc(void *arg)
              rect.right - rect.left, rect.bottom - rect.top,
              SWP_NOZORDER | SWP_FRAMECHANGED);
    }
-   
+
    if (disp->flags & ALLEGRO_FULLSCREEN_WINDOW) {
       bool frameless = true;
       _al_win_set_window_frameless(disp, win_disp->window, frameless);
@@ -1255,12 +1254,12 @@ static void display_thread_proc(void *arg)
    /* Yep, the following is really needed sometimes. */
    /* ... Or is it now that we have dumped DInput? */
    /* <rohannessian> Win98/2k/XP's window forground rules don't let us
-    * make our window the topmost window on launch. This causes issues on 
+    * make our window the topmost window on launch. This causes issues on
     * full-screen apps, as DInput loses input focus on them.
     * We use this trick to force the window to be topmost, when switching
     * to full-screen only. Note that this only works for Win98 and greater.
     * Win95 will ignore our SystemParametersInfo() calls.
-    * 
+    *
     * See http://support.microsoft.com:80/support/kb/articles/Q97/9/25.asp
     * for details.
     */
@@ -1291,7 +1290,7 @@ static void display_thread_proc(void *arg)
 
       if (disp->flags & ALLEGRO_FULLSCREEN) {
          SystemParametersInfo(SPI_SETFOREGROUNDLOCKTIMEOUT,
-              0, (LPVOID)(DWORD)lock_time, SPIF_SENDWININICHANGE | SPIF_UPDATEINIFILE);
+              0, (LPVOID)&lock_time, SPIF_SENDWININICHANGE | SPIF_UPDATEINIFILE);
       }
 #undef SPI_GETFOREGROUNDLOCKTIMEOUT
 #undef SPI_SETFOREGROUNDLOCKTIMEOUT
@@ -1300,8 +1299,8 @@ static void display_thread_proc(void *arg)
 #if 0
    if (disp->flags & ALLEGRO_FULLSCREEN && al_is_mouse_installed()) {
       RAWINPUTDEVICE rid[1];
-      rid[0].usUsagePage = 0x01; 
-      rid[0].usUsage = 0x02; 
+      rid[0].usUsagePage = 0x01;
+      rid[0].usUsage = 0x02;
       rid[0].dwFlags = RIDEV_NOLEGACY;
       rid[0].hwndTarget = 0;
       if (RegisterRawInputDevices(rid, 1, sizeof(rid[0])) == FALSE) {
@@ -1386,10 +1385,13 @@ static bool wgl_resize_helper(ALLEGRO_DISPLAY *d, int width, int height)
       size_t i;
 
       target_bmp = al_get_target_bitmap();
-      if (target_bmp && target_bmp->vt)
-         was_backbuffer = ((ALLEGRO_BITMAP_OGL*)target_bmp)->is_backbuffer;
+      if (target_bmp && target_bmp->vt) {
+         ALLEGRO_BITMAP_EXTRA_OPENGL *extra = target_bmp->parent ?
+            target_bmp->parent->extra : target_bmp->extra;
+         was_backbuffer = extra->is_backbuffer;
+      }
 
-      /* Remeber display bitmaps. */
+      /* Remember display bitmaps. */
       _al_vector_init(&disp_bmps, sizeof(ALLEGRO_BITMAP*));
       for (i = 0; i < _al_vector_size(&d->bitmaps); i++) {
          ALLEGRO_BITMAP **dis = _al_vector_ref(&d->bitmaps, i);
@@ -1476,7 +1478,6 @@ static bool wgl_acknowledge_resize(ALLEGRO_DISPLAY *d)
 {
    WINDOWINFO wi;
    ALLEGRO_DISPLAY_WIN *win_disp = (ALLEGRO_DISPLAY_WIN *)d;
-   ALLEGRO_DISPLAY *ogl_disp = (ALLEGRO_DISPLAY *)d;
    int w, h;
    ALLEGRO_STATE state;
 
@@ -1488,8 +1489,8 @@ static bool wgl_acknowledge_resize(ALLEGRO_DISPLAY *d)
    d->w = w;
    d->h = h;
 
-   setup_gl(d);
-   _al_ogl_resize_backbuffer(ogl_disp->ogl_extras->backbuffer, w, h);
+   _al_ogl_setup_gl(d);
+   _al_ogl_update_render_state(d);
 
    al_store_state(&state, ALLEGRO_STATE_DISPLAY | ALLEGRO_STATE_TARGET_BITMAP);
    al_set_target_backbuffer(d);
@@ -1506,7 +1507,7 @@ static bool wgl_is_compatible_bitmap(ALLEGRO_DISPLAY *display, ALLEGRO_BITMAP *b
     * thus all bitmaps are tied to the display which was current at the time
     * al_create_bitmap was called.
     */
-   return display == bitmap->display;
+   return display == _al_get_bitmap_display(bitmap);
 }
 
 
@@ -1556,7 +1557,6 @@ ALLEGRO_DISPLAY_INTERFACE *_al_display_wgl_driver(void)
    vt.update_display_region = wgl_update_display_region;
    vt.acknowledge_resize = wgl_acknowledge_resize;
    vt.create_bitmap = _al_ogl_create_bitmap;
-   vt.create_sub_bitmap = _al_ogl_create_sub_bitmap;
    vt.get_backbuffer = _al_ogl_get_backbuffer;
    vt.set_target_bitmap = _al_ogl_set_target_bitmap;
    vt.is_compatible_bitmap = wgl_is_compatible_bitmap;
@@ -1571,16 +1571,22 @@ ALLEGRO_DISPLAY_INTERFACE *_al_display_wgl_driver(void)
    vt.set_icons = _al_win_set_display_icons;
    vt.set_window_position = wgl_set_window_position;
    vt.get_window_position = wgl_get_window_position;
+   vt.set_window_constraints = _al_win_set_window_constraints;
+   vt.get_window_constraints = _al_win_get_window_constraints;
+   vt.apply_window_constraints = _al_win_apply_window_constraints;
    vt.set_display_flag = _al_win_set_display_flag;
    vt.set_window_title = _al_win_set_window_title;
+
+   vt.update_render_state = _al_ogl_update_render_state;
    _al_ogl_add_drawing_functions(&vt);
+   _al_win_add_clipboard_functions(&vt);
 
    return &vt;
 }
 
 int _al_wgl_get_num_display_modes(int format, int refresh_rate, int flags)
 {
-   DEVMODE dm;   
+   DEVMODE dm;
    int count = 0;
 
    /* FIXME: Ignoring format.
@@ -1607,7 +1613,7 @@ ALLEGRO_DISPLAY_MODE *_al_wgl_get_display_mode(int index, int format,
                                                int refresh_rate, int flags,
                                                ALLEGRO_DISPLAY_MODE *mode)
 {
-   DEVMODE dm;   
+   DEVMODE dm;
 
    /*
     * FIXME: see the comment in _al_wgl_get_num_display_modes
