@@ -66,6 +66,10 @@ static void display_flags_to_window_styles(int flags,
       *style = WS_POPUP;
       *ex_style = WS_EX_APPWINDOW;
    }
+   else if (flags & ALLEGRO_MAXIMIZED) {
+      *style = WS_OVERLAPPEDWINDOW;
+      *ex_style = WS_EX_APPWINDOW;
+   }
    else if (flags & ALLEGRO_RESIZABLE) {
       *style = WS_OVERLAPPEDWINDOW;
       *ex_style = WS_EX_APPWINDOW | WS_EX_OVERLAPPEDWINDOW;
@@ -76,6 +80,20 @@ static void display_flags_to_window_styles(int flags,
    }
 }
 
+// Clears a window to black
+static void clear_window(HWND window, int w, int h)
+{
+   PAINTSTRUCT ps;
+   HDC hdc;
+
+   hdc = BeginPaint(window, &ps);
+
+   SelectObject(hdc, GetStockObject(DC_BRUSH));
+   SetDCBrushColor(hdc, RGB(0, 0, 0));
+
+   Rectangle(hdc, 0, 0, w, h);
+}
+
 HWND _al_win_create_hidden_window()
 {
    HWND window = CreateWindowEx(0,
@@ -189,6 +207,8 @@ HWND _al_win_create_window(ALLEGRO_DISPLAY *display, int width, int height, int
 
    ShowWindow(my_window, SW_SHOW);
 
+   clear_window(my_window, width, height);
+
    if (!(flags & ALLEGRO_RESIZABLE) && !(flags & ALLEGRO_FULLSCREEN)) {
       /* Make the window non-resizable */
       HMENU menu;
@@ -253,6 +273,8 @@ HWND _al_win_create_faux_fullscreen_window(LPCTSTR devname, ALLEGRO_DISPLAY *dis
 
    ChangeDisplaySettingsEx(devname, &mode, NULL, 0, NULL/*CDS_FULLSCREEN*/);
 
+   clear_window(my_window, width, height);
+
    return my_window;
 }
 
@@ -292,7 +314,11 @@ static void win_generate_resize_event(ALLEGRO_DISPLAY_WIN *win_display)
    if (w == 0 && h == 0 && x == -32000 && y == -32000)
       return;
 
-   if (display->w != w || display->h != h) {
+   /* Always generate resize event when constraints are used.
+    * This is needed because d3d_acknowledge_resize() updates d->w, d->h
+    * before this function will be called.
+    */
+   if (display->use_constraints || display->w != w || display->h != h) {
       _al_event_source_lock(es);
       if (_al_event_source_needs_to_generate_event(es)) {
          ALLEGRO_EVENT event;
@@ -710,6 +736,15 @@ static LRESULT CALLBACK window_callback(HWND hWnd, UINT message,
                (Disables opening menus via the ALT key.) */
             return 0;
          }
+         /* This is used by WM_GETMINMAXINFO to set constraints. */
+         else if ((wParam & 0xfff0) == SC_MAXIMIZE) {
+            d->flags |= ALLEGRO_MAXIMIZED;
+            SetWindowLong(hWnd, GWL_EXSTYLE, WS_EX_APPWINDOW);
+         }
+         else if ((wParam & 0xfff0) == SC_RESTORE) {
+            d->flags &= ~ALLEGRO_MAXIMIZED;
+            SetWindowLong(hWnd, GWL_EXSTYLE, WS_EX_APPWINDOW | WS_EX_OVERLAPPEDWINDOW);
+         }
          break;
       }
       case WM_PAINT: {
@@ -847,7 +882,8 @@ static LRESULT CALLBACK window_callback(HWND hWnd, UINT message,
          _al_event_source_unlock(es);
          return 0;
       case WM_GETMINMAXINFO:
-         {
+         /* Set window constraints only when needed. */
+         if (d->use_constraints) {
             LPMINMAXINFO p_info = (LPMINMAXINFO)lParam;
             RECT wRect;
             RECT cRect;
@@ -871,8 +907,16 @@ static LRESULT CALLBACK window_callback(HWND hWnd, UINT message,
 
                wmin.x = (d->min_w > 0) ? d->min_w + total_border_width : p_info->ptMinTrackSize.x;
                wmin.y = (d->min_h > 0) ? d->min_h + total_border_height : p_info->ptMinTrackSize.y;
-               wmax.x = (d->max_w > 0) ? d->max_w + total_border_width : p_info->ptMaxTrackSize.x;
-               wmax.y = (d->max_h > 0) ? d->max_h + total_border_height : p_info->ptMaxTrackSize.y;
+
+               /* don't use max_w & max_h constraints when window maximized */
+               if (d->flags & ALLEGRO_MAXIMIZED) {
+                  wmax.x = p_info->ptMaxTrackSize.x;
+                  wmax.y = p_info->ptMaxTrackSize.y;
+               }
+               else {
+                  wmax.x = (d->max_w > 0) ? d->max_w + total_border_width : p_info->ptMaxTrackSize.x;
+                  wmax.y = (d->max_h > 0) ? d->max_h + total_border_height : p_info->ptMaxTrackSize.y;
+               }
 
                p_info->ptMinTrackSize = wmin;
                p_info->ptMaxTrackSize = wmax;
@@ -890,7 +934,18 @@ static LRESULT CALLBACK window_callback(HWND hWnd, UINT message,
             }
             d->flags &= ~ALLEGRO_MAXIMIZED;
             if (wParam == SIZE_MAXIMIZED) {
-                d->flags |= ALLEGRO_MAXIMIZED;
+               d->flags |= ALLEGRO_MAXIMIZED;
+            }
+         }
+         else if (d->use_constraints) {
+            /* al_apply_window_constraints() resizes a window if the current
+             * width & height do not fit in the constraint's range.
+             * We have to create the resize event, so the application could
+             * redraw its content.
+             */
+            if (!resize_postponed) {
+               resize_postponed = true;
+               _beginthread(postpone_thread_proc, 0, (void *)d);
             }
          }
          return 0;
@@ -1202,7 +1257,7 @@ bool _al_win_set_display_flag(ALLEGRO_DISPLAY *display, int flag, bool onoff)
             int pos_y = 0;
             WINDOWINFO wi;
             int bw, bh;
-            
+
             // Unset the topmost flag
             SetWindowPos(win_display->window, HWND_NOTOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
 
@@ -1218,9 +1273,13 @@ bool _al_win_set_display_flag(ALLEGRO_DISPLAY *display, int flag, bool onoff)
                win_display->window, HWND_TOP, pos_x-bw/2, pos_y-bh/2, 0, 0, SWP_NOSIZE
             );
          }
+
          // Show the window again
          SetWindowPos(win_display->window, 0, 0, 0, 0, 0, SWP_SHOWWINDOW | SWP_NOSIZE | SWP_NOZORDER | SWP_NOMOVE);
 
+         // Clear the window to black
+         clear_window(win_display->window, display->w, display->h);
+
          ASSERT(!!(display->flags & ALLEGRO_FULLSCREEN_WINDOW) == onoff);
          return true;
       case ALLEGRO_MAXIMIZED:
@@ -1249,18 +1308,20 @@ void _al_win_set_window_title(ALLEGRO_DISPLAY *display, const char *title)
 bool _al_win_set_window_constraints(ALLEGRO_DISPLAY *display,
    int min_w, int min_h, int max_w, int max_h)
 {
-   ALLEGRO_DISPLAY_WIN *win_display = (ALLEGRO_DISPLAY_WIN *)display;
-   win_display->display.min_w = min_w;
-   win_display->display.min_h = min_h;
-   win_display->display.max_w = max_w;
-   win_display->display.max_h = max_h;
-
-   /* Resize so constraints can take effect. */
-   al_resize_display(display, display->w, display->h);
+   display->min_w = min_w;
+   display->min_h = min_h;
+   display->max_w = max_w;
+   display->max_h = max_h;
 
    return true;
 }
 
+void _al_win_apply_window_constraints(ALLEGRO_DISPLAY *display, bool onoff)
+{
+   if (!(display->flags & ALLEGRO_MAXIMIZED))
+      al_resize_display(display, display->w, display->h);
+}
+
 void _al_win_post_create_window(ALLEGRO_DISPLAY *display)
 {
    /* Ideally the d3d/wgl window creation would already create the
@@ -1346,7 +1407,7 @@ bool al_win_add_window_callback(ALLEGRO_DISPLAY *display,
 {
    ALLEGRO_DISPLAY_WIN *win_display = (ALLEGRO_DISPLAY_WIN *) display;
    ALLEGRO_DISPLAY_WIN_CALLBACK *ptr;
-   
+
    if (!display || !callback) {
       return false;
    }
@@ -1374,7 +1435,7 @@ bool al_win_remove_window_callback(ALLEGRO_DISPLAY *display,
    LPARAM lparam, LRESULT *result, void *userdata), void *userdata)
 {
    ALLEGRO_DISPLAY_WIN *win_display = (ALLEGRO_DISPLAY_WIN *) display;
-   
+
    if (display && callback) {
       size_t i;
       for (i = 0; i < _al_vector_size(&win_display->msg_callbacks); ++i) {
