@@ -17,10 +17,152 @@ extern int _Xdebug; /* part of Xlib */
 #include "allegro5/platform/aintunix.h"
 #include "allegro5/platform/aintxglx.h"
 
+#include "allegro5/allegro_x.h"
+
+#include "xicon.h"
+
 ALLEGRO_DEBUG_CHANNEL("system")
 
 static ALLEGRO_SYSTEM_INTERFACE *xglx_vt;
 
+char **x11_xpm = NULL;
+
+#ifdef ALLEGRO_XWINDOWS_WITH_XPM
+#include <stdio.h>
+#include "icon.xpm"
+
+static bool x11_xpm_set;
+static int x11_xpm_rows;
+
+static char **bitmap_to_xpm(ALLEGRO_BITMAP *bitmap, int *nrows_ret)
+{
+   _AL_VECTOR v;
+   int w, h, x, y;
+   int ncolors, nrows;
+   char **xpm;
+   char buf[100];
+   int i;
+
+   ALLEGRO_LOCKED_REGION *lr = al_lock_bitmap(bitmap, ALLEGRO_PIXEL_FORMAT_ABGR_8888_LE, ALLEGRO_LOCK_READONLY);
+   if (lr == NULL)
+      return NULL;
+
+   _al_vector_init(&v, sizeof(uint32_t));
+
+   w = al_get_bitmap_width(bitmap);
+   h = al_get_bitmap_height(bitmap);
+
+   for (y = 0; y < h; y++) {
+      for (x = 0; x < w; x++) {
+         uint32_t c = *(uint32_t *)((((char *)lr->data) + lr->pitch * y + x * 4));
+         int alpha = (c >> 24) & 0xff;
+         if (alpha != 255) {
+                 c = 0;
+         }
+         int sz = _al_vector_size(&v);
+         bool found = false;
+         for (i = 0; i < sz; i++) {
+            if (*((uint32_t *)_al_vector_ref(&v, i)) == c) {
+               found = true;
+               break;
+            }
+         }
+         if (found == false) {
+            uint32_t *p = _al_vector_alloc_back(&v);
+            *p = c;
+         }
+      }
+   }
+
+   ncolors = _al_vector_size(&v);
+   nrows = 2 + ncolors + h;
+
+   xpm = malloc(nrows * sizeof(char *));
+   if (xpm == NULL)
+      return NULL;
+
+   snprintf(buf, 100, "%d %d %d 8", w, h, ncolors + 1);
+
+   xpm[0] = strdup(buf);
+
+   xpm[1] = strdup("00000000\tc None");
+
+   for (i = 0; i < ncolors; i++) {
+        uint32_t c = *((uint32_t *)_al_vector_ref(&v, i));
+        int r = c & 0xff;
+        int g = (c >> 8) & 0xff;
+        int b = (c >> 16) & 0xff;
+        snprintf(buf, 100, "%08x\tc #%02x%02x%02x", i+1, r, g, b);
+        xpm[i+2] = strdup(buf);
+   }
+
+   for (y = 0; y < h; y++) {
+        int row = y + 2 + ncolors;
+        xpm[row] = malloc(8 * w + 1);
+        xpm[row][8 * w] = 0;
+        uint32_t *p = (uint32_t *)(((char *)lr->data) + lr->pitch * y);
+        for (x = 0; x < w; x++) {
+                uint32_t pixel = *p;
+                int alpha = (pixel >> 24) & 0xff;
+                if (alpha != 255) {
+                   snprintf(buf, 100, "%s", "00000000");
+                }
+                else {
+                   for (i = 0; i < (int)_al_vector_size(&v); i++) {
+                      uint32_t pixel2 = *((uint32_t *)_al_vector_ref(&v, i));
+                      if (pixel == pixel2)
+                         break;
+                   }
+                   snprintf(buf, 100, "%08x", i+1);
+                }
+                for (i = 0; i < 8; i++) {
+                        xpm[row][8*x+i] = buf[i];
+                }
+                p++;
+        }
+   }
+
+   _al_vector_free(&v);
+
+   *nrows_ret = nrows;
+
+   al_unlock_bitmap(bitmap);
+
+   // debug
+   /*
+   for (i = 0; i < nrows; i++) {
+      printf("%s\n", xpm[i]);
+   }
+   */
+
+   return xpm;
+}
+#endif
+
+/* Function: al_x_set_initial_icon
+ */
+bool al_x_set_initial_icon(ALLEGRO_BITMAP *bitmap)
+{
+#ifdef ALLEGRO_XWINDOWS_WITH_XPM
+   if (x11_xpm_set) {
+      int i;
+      for (i = 0; i < x11_xpm_rows; i++) {
+         free(x11_xpm[i]);
+      }
+      free(x11_xpm);
+      x11_xpm_set = false;
+   }
+   x11_xpm = bitmap_to_xpm(bitmap, &x11_xpm_rows);
+   if (x11_xpm == NULL)
+      return false;
+   x11_xpm_set = true;
+   return true;
+#else
+   (void)bitmap;
+   return false;
+#endif
+}
+
 static ALLEGRO_SYSTEM *xglx_initialize(int flags)
 {
    Display *x11display;
@@ -86,6 +228,23 @@ static ALLEGRO_SYSTEM *xglx_initialize(int flags)
       ALLEGRO_INFO("events thread spawned.\n");
    }
 
+   const char *binding = al_get_config_value(al_get_system_config(),
+         "keyboard", "toggle_mouse_grab_key");
+   if (binding) {
+      s->toggle_mouse_grab_keycode = _al_parse_key_binding(binding,
+         &s->toggle_mouse_grab_modifiers);
+      if (s->toggle_mouse_grab_keycode) {
+         ALLEGRO_DEBUG("Toggle mouse grab key: '%s'\n", binding);
+      }
+      else {
+         ALLEGRO_WARN("Cannot parse key binding '%s'\n", binding);
+      }
+   }
+
+#ifdef ALLEGRO_XWINDOWS_WITH_XPM
+   x11_xpm = icon_xpm;
+#endif
+
    return &s->system;
 }
 
@@ -133,27 +292,6 @@ static void xglx_shutdown_system(void)
 
 static ALLEGRO_DISPLAY_INTERFACE *xglx_get_display_driver(void)
 {
-   ALLEGRO_SYSTEM_XGLX *system = (ALLEGRO_SYSTEM_XGLX *)al_get_system_driver();
-
-   /* Look up the toggle_mouse_grab_key binding.  This isn't such a great place
-    * to do it, but the config file is not available until after the system driver
-    * is initialised.
-    */
-   if (!system->toggle_mouse_grab_keycode) {
-      const char *binding = al_get_config_value(al_get_system_config(),
-         "keyboard", "toggle_mouse_grab_key");
-      if (binding) {
-         system->toggle_mouse_grab_keycode = _al_parse_key_binding(binding,
-            &system->toggle_mouse_grab_modifiers);
-         if (system->toggle_mouse_grab_keycode) {
-            ALLEGRO_DEBUG("Toggle mouse grab key: '%s'\n", binding);
-         }
-         else {
-            ALLEGRO_WARN("Cannot parse key binding '%s'\n", binding);
-         }
-      }
-   }
-
    return _al_display_xglx_driver();
 }
 
