@@ -70,6 +70,7 @@ int               verbose = 0;
 int               total_tests = 0;
 int               passed_tests = 0;
 int               failed_tests = 0;
+int               skipped_tests = 0;
 
 #define streq(a, b)  (0 == strcmp((a), (b)))
 
@@ -262,11 +263,14 @@ static bool get_bool(char const *value)
 static ALLEGRO_COLOR get_color(char const *value)
 {
    int r, g, b, a;
+   float rf, gf, bf;
 
    if (sscanf(value, "#%02x%02x%02x%02x", &r, &g, &b, &a) == 4)
       return al_map_rgba(r, g, b, a);
    if (sscanf(value, "#%02x%02x%02x", &r, &g, &b) == 3)
       return al_map_rgb(r, g, b);
+   if (sscanf(value, "%f/%f/%f", &rf, &gf, &bf) == 3)
+      return al_map_rgb_f(rf, gf, bf);
    return al_color_name(value);
 }
 
@@ -932,7 +936,7 @@ static void check_similarity(ALLEGRO_CONFIG const *cfg,
 }
 
 static void do_test(ALLEGRO_CONFIG *cfg, char const *testname,
-   ALLEGRO_BITMAP *target, int bmp_type, bool reliable)
+   ALLEGRO_BITMAP *target, int bmp_type, bool reliable, bool do_check_hash)
 {
 #define MAXBUF    80
 
@@ -1439,6 +1443,11 @@ static void do_test(ALLEGRO_CONFIG *cfg, char const *testname,
          set_config_float(cfg, testname, lval, result);
          continue;
       }
+      if (SCANLVAL("round", 1)) {
+         int result  = round(F(0));
+         set_config_int(cfg, testname, lval, result);
+         continue;
+      }
 
       /* Dynamical variable initialisation, needed to properly initialize
        * variables (5.1)*/
@@ -1479,17 +1488,28 @@ static void do_test(ALLEGRO_CONFIG *cfg, char const *testname,
          continue;
       }
 
+      if (SCANLVAL("al_color_distance_ciede2000", 2)) {
+         float d = al_color_distance_ciede2000(C(0), C(1));
+         set_config_float(cfg, testname, lval, d);
+         continue;
+      }
+      if (SCANLVAL("al_color_lab", 3)) {
+         ALLEGRO_COLOR rgb = al_color_lab(F(0), F(1), F(2));
+         char hex[100];
+         sprintf(hex, "%f/%f/%f", rgb.r, rgb.g, rgb.b);
+         al_set_config_value(cfg, testname, lval, hex);
+         continue;
+      }
+
       fatal_error("statement didn't scan: %s", stmt);
    }
 
    al_set_new_bitmap_format(ALLEGRO_PIXEL_FORMAT_ANY_WITH_ALPHA);
 
-   if (bmp_type == SW) {
-      al_set_new_bitmap_flags(ALLEGRO_MEMORY_BITMAP);
+   if (do_check_hash) {
       check_hash(cfg, testname, target, bmp_type);
    }
    else {
-      al_set_new_bitmap_flags(ALLEGRO_VIDEO_BITMAP);
       check_similarity(cfg, testname, target, membuf, bmp_type, reliable);
    }
 
@@ -1538,15 +1558,27 @@ static void sw_hw_test(ALLEGRO_CONFIG *cfg, char const *testname)
 {
    int old_failed_tests = failed_tests;
    bool reliable;
+   char const *hw_only_str = al_get_config_value(cfg, testname, "hw_only");
+   char const *sw_only_str = al_get_config_value(cfg, testname, "sw_only");
+   bool hw_only = hw_only_str && get_bool(hw_only_str);
+   bool sw_only = sw_only_str && get_bool(sw_only_str);
 
    al_set_new_bitmap_flags(ALLEGRO_MEMORY_BITMAP);
-   do_test(cfg, testname, membuf, SW, true);
+   if (!hw_only) {
+      do_test(cfg, testname, membuf, SW, true, true);
+   }
 
    reliable = (failed_tests == old_failed_tests);
 
+   if (sw_only) return;
+
    if (display) {
       al_set_new_bitmap_flags(ALLEGRO_VIDEO_BITMAP);
-      do_test(cfg, testname, al_get_backbuffer(display), HW, reliable);
+      do_test(cfg, testname, al_get_backbuffer(display), HW, reliable, hw_only);
+   } else if (hw_only) {
+      printf("WARNING: Skipping hardware-only test due to the --no-display flag: %s\n",
+             testname);
+      skipped_tests++;
    }
 }
 
@@ -1811,6 +1843,7 @@ int main(int _argc, char *_argv[])
    printf("total tests:  %d\n", total_tests);
    printf("passed tests: %d\n", passed_tests);
    printf("failed tests: %d\n", failed_tests);
+   printf("skipped tests: %d\n", skipped_tests);
    printf("\n");
 
    return !!failed_tests;
